@@ -156,40 +156,49 @@ void Bitmap::copyPixels(const Bitmap & Orig)
     if (&Orig == this) {
         return;
     }
-    if (Orig.getPixelFormat() == m_PF) {
-        const unsigned char * pSrc = Orig.getPixels();
-        unsigned char * pDest = m_pBits;
-        int Height = min(Orig.getSize().y, m_Size.y);
-        int Width = min(Orig.getSize().x, m_Size.x);
-        int LineLen = Width*getBytesPerPixel();
-        for (int y=0; y<Height; ++y) {
-            memcpy(pDest, pSrc, LineLen);
-            pDest += m_Stride;
-            pSrc += Orig.getStride();
+    if (Orig.getPixelFormat() == YCbCr422 || Orig.getPixelFormat() == YCbCr411) {
+        if (m_PF == B8G8R8X8) {
+            YCbCrtoBGR(Orig);
+        } else {
+            Bitmap TempBmp(getSize(), B8G8R8X8, "TempColorConversion");
+            TempBmp.YCbCrtoBGR(Orig);
+            copyPixels(TempBmp);
         }
     } else {
-        switch(m_PF) {
-            case B8G8R8A8:
-            case B8G8R8X8:
-            case A8B8G8R8:
-            case X8B8G8R8:
-            case R8G8B8A8:
-            case R8G8B8X8:
-            case A8R8G8B8:
-            case X8R8G8B8:
-                createTrueColorCopy<Pixel32>(*this, Orig);
-                break;
-            case B8G8R8:
-            case R8G8B8:
-                createTrueColorCopy<Pixel24>(*this, Orig);
-                break;
-            case B5G6R5:
-            case R5G6B5:
-                createTrueColorCopy<Pixel16>(*this, Orig);
-                break;
-            default:
-                // Unimplemented conversion.
-                assert(false);
+        if (Orig.getPixelFormat() == m_PF) {
+            const unsigned char * pSrc = Orig.getPixels();
+            unsigned char * pDest = m_pBits;
+            int Height = min(Orig.getSize().y, m_Size.y);
+            int LineLen = getLineLen();
+            for (int y=0; y<Height; ++y) {
+                memcpy(pDest, pSrc, LineLen);
+                pDest += m_Stride;
+                pSrc += Orig.getStride();
+            }
+        } else {
+            switch(m_PF) {
+                case B8G8R8A8:
+                case B8G8R8X8:
+                case A8B8G8R8:
+                case X8B8G8R8:
+                case R8G8B8A8:
+                case R8G8B8X8:
+                case A8R8G8B8:
+                case X8R8G8B8:
+                    createTrueColorCopy<Pixel32>(*this, Orig);
+                    break;
+                case B8G8R8:
+                case R8G8B8:
+                    createTrueColorCopy<Pixel24>(*this, Orig);
+                    break;
+                case B5G6R5:
+                case R5G6B5:
+                    createTrueColorCopy<Pixel16>(*this, Orig);
+                    break;
+                default:
+                    // Unimplemented conversion.
+                    assert(false);
+            }
         }
     }
 }
@@ -322,10 +331,14 @@ std::string Bitmap::getPixelFormatString(PixelFormat PF)
             return "X8R8G8B8";
         case I8:
             return "I8";
+        case YCbCr411:
+            return "YCbCr411";
         case YCbCr422:
             return "YCbCr422";
         case YCbCr420p:
             return "YCbCr420p";
+        case YCbCrJ420p:
+            return "YCbCrJ420p";
         default:
             return "Unknown";
     }
@@ -400,6 +413,15 @@ int Bitmap::getBytesPerPixel(PixelFormat PF)
     }
 }
 
+int Bitmap::getLineLen() const
+{
+    if (m_PF == YCbCr411) {
+        return int(m_Size.x*1.5);
+    } else {
+        return m_Size.x*getBytesPerPixel();
+    }
+}
+
 int Bitmap::getMemNeeded() const
 {
     // This assumes a positive value for stride.
@@ -423,7 +445,7 @@ bool Bitmap::operator ==(const Bitmap & otherBmp)
 
     const unsigned char * pSrc = otherBmp.getPixels();
     unsigned char * pDest = m_pBits;
-    int LineLen = getSize().x*getBytesPerPixel();
+    int LineLen = getLineLen();
     for (int y=0; y<getSize().y; ++y) {
         switch(m_PF) {
             case R8G8B8X8:
@@ -503,9 +525,8 @@ int Bitmap::getNumDifferentPixels(const Bitmap & otherBmp)
 {
     // We allow Name, Stride and bOwnsBits to be different here, since we're looking for
     // equal value only.
-    if (m_Size != otherBmp.m_Size || m_PF != otherBmp.m_PF)
-    {
-        return false;
+    if (m_Size != otherBmp.m_Size || m_PF != otherBmp.m_PF) {
+        return m_Size.x*m_Size.y;
     }
 
     BitmapPtr pTempBmp(new Bitmap(*this));
@@ -596,7 +617,7 @@ void Bitmap::initWithData(unsigned char * pBits, int Stride, bool bCopyBits)
 void Bitmap::allocBits()
 {
 //    cerr << "Bitmap::allocBits():" << m_Size <<  endl;
-    
+    m_Stride = getLineLen();
     if (m_PF == YCbCr422) {
         if (m_Size.x%2 == 1) {
             AVG_TRACE(Logger::WARNING, "Odd size for YCbCr bitmap.");
@@ -606,17 +627,135 @@ void Bitmap::allocBits()
             AVG_TRACE(Logger::WARNING, "Odd size for YCbCr bitmap.");
             m_Size.y++;
         }
-        m_Stride = m_Size.x*getBytesPerPixel();
         //XXX: We allocate more than nessesary here because ffmpeg seems to
         // overwrite memory after the bits - probably during yuv conversion.
         // Yuck.
         m_pBits = new unsigned char[(m_Stride+1)*(m_Size.y+1)];
     } else {
-        m_Stride = m_Size.x*getBytesPerPixel();
         m_pBits = new unsigned char[m_Stride*m_Size.y];
     }
 }
 
+inline void YUVtoBGR32Pixel(Pixel32* pDest, int y, int u, int v)
+{
+    // u = Cb, v = Cr
+    int u1 = u - 128;
+    int v1 = v - 128;
+    int tempy = 298*(y-16);
+    int b = (tempy + 516 * u1           ) >> 8;
+    int g = (tempy - 100 * u1 - 208 * v1) >> 8;
+    int r = (tempy            + 409 * v1) >> 8;
+
+    if (b<0) b = 0;
+    if (b>255) b= 255;
+    if (g<0) g = 0;
+    if (g>255) g= 255;
+    if (r<0) r = 0;
+    if (r>255) r= 255;
+    pDest->set(b,g,r,255);
+}
+
+void YUV422toBGR32Line(const unsigned char* pSrcLine, Pixel32 * pDestLine, int Width)
+{
+    Pixel32 * pDestPixel = pDestLine;
+    
+    // We need the previous and next values to interpolate between the
+    // sampled u and v values.
+    int v = *(pSrcLine+2);
+    int v0; // Previous v
+    int u;
+    int u1; // Next u;
+    const unsigned char * pSrcPixels = pSrcLine;
+
+    for (int x = 0; x < Width/2-1; x++) {
+        // Two pixels at a time.
+        // Source format is UYVY.
+        u = pSrcPixels[0];
+        v0 = v;
+        v = pSrcPixels[2];
+        u1 = pSrcPixels[4];
+
+        YUVtoBGR32Pixel(pDestPixel, pSrcPixels[1], u, (v0+v)/2);
+        YUVtoBGR32Pixel(pDestPixel+1, pSrcPixels[3], (u+u1)/2, v);
+
+        pSrcPixels+=4;
+        pDestPixel+=2;
+    }
+    // Last pixels.
+    u = pSrcPixels[0];
+    v0 = v;
+    v = pSrcPixels[2];
+    YUVtoBGR32Pixel(pDestPixel, pSrcPixels[1], u, v0/2+v/2);
+    YUVtoBGR32Pixel(pDestPixel+1, pSrcPixels[3], u, v);
+}
+ 
+void YUV411toBGR32Line(const unsigned char* pSrcLine, Pixel32 * pDestLine, int Width)
+{
+    Pixel32 * pDestPixel = pDestLine;
+    
+    // We need the previous and next values to interpolate between the
+    // sampled u and v values.
+    int v = *(pSrcLine+3);
+    int v0; // Previous v
+    int v1; // Next v;
+    int u;
+    int u1; // Next u;
+    const unsigned char * pSrcPixels = pSrcLine;
+
+    for (int x = 0; x < Width/4; x++) {
+        // Four pixels at a time.
+        // Source format is UYYVYY.
+        u = pSrcPixels[0];
+        v0 = v;
+        v = pSrcPixels[3];
+
+        if (x < Width/4-1) {
+            u1 = pSrcPixels[6];
+            v1 = pSrcPixels[9];
+        } else {
+            u1 = u;
+            v1 = v;
+        }
+
+        YUVtoBGR32Pixel(pDestPixel, pSrcPixels[1], u, v0/2+v/2);
+        YUVtoBGR32Pixel(pDestPixel+1, pSrcPixels[2], (u*3)/4+u1/4, v0/4+(v*3)/4);
+        YUVtoBGR32Pixel(pDestPixel+2, pSrcPixels[4], u/2+u1/2, v);
+        YUVtoBGR32Pixel(pDestPixel+3, pSrcPixels[5], u/4+(u1*3)/4, (v*3)/4+v1/4);
+
+        pSrcPixels+=6;
+        pDestPixel+=4;
+    }
+}
+
+void Bitmap::YCbCrtoBGR(const Bitmap& Orig)
+{
+    assert(m_PF==B8G8R8X8);
+    const unsigned char * pSrc = Orig.getPixels();
+    Pixel32 * pDest = (Pixel32*)m_pBits;
+    int Height = min(Orig.getSize().y, m_Size.y);
+    int Width = min(Orig.getSize().x, m_Size.x);
+    int StrideInPixels = m_Stride/getBytesPerPixel();
+    switch(Orig.m_PF) {
+        case YCbCr422:
+            for (int y=0; y<Height; ++y) {
+                YUV422toBGR32Line(pSrc, pDest, Width);
+                pDest += StrideInPixels;
+                pSrc += Orig.getStride();
+            }
+            break;
+        case YCbCr411:
+            for (int y=0; y<Height; ++y) {
+                YUV411toBGR32Line(pSrc, pDest, Width);
+                pDest += StrideInPixels;
+                pSrc += Orig.getStride();
+            }
+            break;
+        default:
+            // This routine shouldn't be called with other pixel formats.
+            assert(false);
+    }
+}
+
 template<class DestPixel, class SrcPixel>
 void createTrueColorCopy(Bitmap& Dest, const Bitmap & Src)
 {
