@@ -50,25 +50,27 @@ namespace avg {
 
 Camera::Camera ()
 #ifdef AVG_ENABLE_1394
-    : m_sDevice("Default"),
+    : m_sDevice(""),
       m_FrameRate(15),
       m_sMode("640x480_RGB"),
-      m_FWHandle(0)
+      m_FWHandle(0),
+      m_bCameraAvailable(false)
 #else
     : m_sDevice("Camera disabled"),
-      m_FrameRate(30),
+      m_FrameRate(15),
       m_sMode("---")
 #endif
 {
 }
 
-Camera::Camera (const xmlNodePtr xmlNode, DivNode * pParent)
-    : VideoBase(xmlNode, pParent),
+Camera::Camera (const xmlNodePtr xmlNode, Player * pPlayer)
+    : VideoBase(xmlNode, pPlayer),
 #ifdef AVG_ENABLE_1394
       m_sDevice(""),
       m_FrameRate(15),
       m_sMode("640x480_RGB"),
-      m_FWHandle(0)
+      m_FWHandle(0),
+      m_bCameraAvailable(false)
 #else
       m_sDevice("Camera disabled"),
       m_FrameRate(15),
@@ -90,10 +92,10 @@ Camera::Camera (const xmlNodePtr xmlNode, DivNode * pParent)
 
 Camera::~Camera ()
 {
+    close();
 }
 
-void Camera::init (DisplayEngine * pEngine, DivNode * pParent,
-        Player * pPlayer)
+void Camera::setDisplayEngine(DisplayEngine * pEngine)
 {
 #ifdef AVG_ENABLE_1394
     if (m_FrameRate == 1.875) {
@@ -140,7 +142,7 @@ void Camera::init (DisplayEngine * pEngine, DivNode * pParent,
     AVG_TRACE(Logger::ERROR,
             "Unable to set up camera. Camera support not compiled.");
 #endif
-    VideoBase::init(pEngine, pParent, pPlayer);
+    VideoBase::setDisplayEngine(pEngine);
 }
 
 string Camera::getTypeStr ()
@@ -238,11 +240,11 @@ double Camera::getFPS()
 
 void Camera::open(int* pWidth, int* pHeight)
 {
-    int CaptureFormat = 0;
     *pWidth = getNativeSize().x;
     *pHeight = getNativeSize().y;
     
 #ifdef AVG_ENABLE_1394
+    int CaptureFormat = 0;
     // TODO: Support other resolutions.
     switch(m_Mode) {
         case MODE_640x480_YUV422:
@@ -306,15 +308,7 @@ void Camera::open(int* pWidth, int* pHeight)
         fatalError(string("Failed to not make camera root node (Node: ")+
                 getID() + ").");
     }
-
-    unsigned int channel;
-    unsigned int speed;
     int err;
-    err = dc1394_get_iso_channel_and_speed(m_FWHandle,
-                m_Camera.node, &channel, &speed);
-    checkDC1394Error(err,
-            "Unable to get the firewire camera iso channel number.");
-
     err = dc1394_get_camera_feature_set(m_FWHandle,
                 m_Camera.node, &m_FeatureSet);
     checkDC1394Error(err,
@@ -326,7 +320,7 @@ void Camera::open(int* pWidth, int* pHeight)
         pDeviceFileName = m_sDevice.c_str();
     }
     err = dc1394_dma_setup_capture(m_FWHandle, m_Camera.node,
-                channel+1, CaptureFormat, m_Mode,
+                1, CaptureFormat, m_Mode,
                 SPEED_400, m_FrameRateConstant, NUM_BUFFERS, DROP_FRAMES, 
                 pDeviceFileName, &m_Camera);
     if (err != DC1394_SUCCESS) {
@@ -390,9 +384,11 @@ void Camera::close()
 {
 #ifdef AVG_ENABLE_1394
     if (m_bCameraAvailable) {
+//        dc1394_stop_iso_transmission(m_FWHandle, m_Camera.node);
         dc1394_dma_unlisten(m_FWHandle, &m_Camera);
         dc1394_dma_release_camera(m_FWHandle, &m_Camera);
         dc1394_destroy_handle(m_FWHandle);
+        m_bCameraAvailable = false;
     }
 #endif
 }
@@ -415,7 +411,7 @@ void Camera::fatalError(const string & sMsg)
 
 static ProfilingZone CameraProfilingZone("    Camera::render");
 static ProfilingZone CameraUploadProfilingZone("      Camera tex download");
-static ProfilingZone CameraYUVConvertProfilingZone("      Camera YUV conversion");
+static ProfilingZone CameraConvertProfilingZone("      Camera format conversion");
 
 bool Camera::renderToSurface(ISurface * pSurface)
 {
@@ -431,65 +427,62 @@ bool Camera::renderToSurface(ISurface * pSurface)
 #endif
 */
             // New frame available
-            switch (m_Mode) {
-                case MODE_640x480_YUV422:
-                case MODE_1024x768_YUV422:
-                    {
-                        BitmapPtr pBmp = pSurface->lockBmp();
-                        ScopeTimer Timer(CameraYUVConvertProfilingZone);
-                        YUV422toBGR32((unsigned char *)(m_Camera.capture_buffer), pBmp);
-                    }
-                    break;
-                case MODE_640x480_YUV411:
-                    {
-                        BitmapPtr pBmp = pSurface->lockBmp();
-                        ScopeTimer Timer(CameraYUVConvertProfilingZone);
-                        YUV411toBGR32((unsigned char *)(m_Camera.capture_buffer), pBmp);
-                    }
-                    break;
-                case MODE_640x480_RGB:
-                case MODE_1024x768_RGB:
-                    {
-/*
-#ifdef AVG_ENABLE_GL
-                        if (pOGLSurface) {
-                            pOGLSurface->createFromBits(getNativeSize(), R8G8B8,
-                                    (unsigned char *)(m_Camera.capture_buffer), 
-                                    getNativeSize().x*3);
-                        } else {
-#endif
-*/
-                            BitmapPtr pBmp = pSurface->lockBmp();
-                            unsigned char * pPixels = pBmp->getPixels();
-
+            BitmapPtr pBmp = pSurface->lockBmp();
+            {
+                ScopeTimer Timer(CameraConvertProfilingZone);
+                switch (m_Mode) {
+                    case MODE_640x480_YUV422:
+                    case MODE_1024x768_YUV422:
+                        {
+                            Bitmap TempBmp(pBmp->getSize(), YCbCr422, 
+                                    (unsigned char *)(m_Camera.capture_buffer),
+                                    getNativeSize().x*2, false, "TempCameraBmp");
+                            pBmp->copyPixels(TempBmp);
+                        }
+                        break;
+                    case MODE_640x480_YUV411:
+                        {
+                            Bitmap TempBmp(pBmp->getSize(), YCbCr411, 
+                                    (unsigned char *)(m_Camera.capture_buffer),
+                                    (int)(getNativeSize().x*1.5), false, "TempCameraBmp");
+                            pBmp->copyPixels(TempBmp);
+                        }
+                        break;
+                    case MODE_640x480_RGB:
+                    case MODE_1024x768_RGB:
+                        {
                             if (getEngine()->hasRGBOrdering()) {
                                 AVG_TRACE(Logger::ERROR,
                                         "Wrong engine rgb order for camera. Aborting.");
                             } else {
-                                for (int y = 0; y < pBmp->getSize().y; y++) {
-                                    unsigned char * pDestLine = 
-                                        pPixels+y*pBmp->getStride();
-                                    unsigned char * pSrcLine = (unsigned char*)
-                                        m_Camera.capture_buffer+3*pBmp->getSize().x*y;
-                                    for (int x = 0; x<pBmp->getSize().x; x++) {
-                                        pDestLine[x*4] = pSrcLine[x*3+2];
-                                        pDestLine[x*4+1] = pSrcLine[x*3+1];
-                                        pDestLine[x*4+2] = pSrcLine[x*3];
-                                        pDestLine[x*4+3] = 0xFF;
+                                unsigned char * pSrcLine = (unsigned char*)
+                                        m_Camera.capture_buffer;
+                                int SrcStride = 3*pBmp->getSize().x;
+                                unsigned char * pDestLine = pBmp->getPixels();
+                                int DestStride = pBmp->getStride();
+                                int Height = pBmp->getSize().y;
+                                int Width = pBmp->getSize().x;
+                                for (int y = 0; y < Height; y++) {
+                                    unsigned char * pDest = pDestLine;
+                                    unsigned char * pSrc = pSrcLine;
+                                    for (int x = 0; x<Width; x++) {
+                                        *pDest++ = *(pSrc+2);
+                                        *pDest++ = *(pSrc+1);
+                                        *pDest++ = *pSrc;
+                                        *pDest++ = 0xFF;
+                                        pSrc += 3;
                                     }
+                                    pSrcLine += SrcStride;
+                                    pDestLine += DestStride;
                                 }
                             }
-/*
-#ifdef AVG_ENABLE_GL
                         }
-#endif
-*/
-                    }
-                    break;
-                default:
-                    AVG_TRACE(Logger::WARNING,
-                            "Illegal Mode in renderToSurface");
-                    break;
+                        break;
+                    default:
+                        AVG_TRACE(Logger::WARNING,
+                                "Illegal Mode in renderToSurface");
+                        break;
+                }
             }
             pSurface->unlockBmps();
             {
@@ -533,121 +526,6 @@ bool Camera::canRenderToBackbuffer(int BitsPerPixel)
 }
 
 #ifdef AVG_ENABLE_1394
-inline void YUVtoBGR32Pixel(Pixel32* pDest, 
-        unsigned char y, unsigned char u, unsigned char v)
-{
-//    pDest->Set(y,y,y);
-    // u = Cb, v = Cr
-    int y1 = y - 16;
-    int u1 = u - 128;
-    int v1 = v - 128;
-    int b = (298 * y1 + 516 * u1           ) >> 8;
-    int g = (298 * y1 - 100 * u1 - 208 * v1) >> 8;
-    int r = (298 * y1            + 409 * v1) >> 8;
-
-    if (b<0) b = 0;
-    if (b>255) b= 255;
-    if (g<0) g = 0;
-    if (g>255) g= 255;
-    if (r<0) r = 0;
-    if (r>255) r= 255;
-    pDest->set(b,g,r);
-}
-
-void Camera::YUV411toBGR32Line(unsigned char* pSrc, int y, Pixel32 * pDestLine)
-{
-    Pixel32 * pDestPixel = pDestLine;
-    int width = getMediaWidth();
-    // We need the previous and next values to interpolate between the
-    // sampled u and v values.
-    unsigned char v = *(pSrc+y*(width*3)/2+3);
-    unsigned char v0; // Previous v
-    unsigned char v1; // Next v;
-    unsigned char u;
-    unsigned char u1; // Next u;
-    unsigned char * pSrcPixels = pSrc+y*(width*3)/2;
-
-    for (int x = 0; x < width/4; x++) {
-        // Four pixels at a time.
-        // Source format is UYYVYY.
-        u = pSrcPixels[0];
-        v0 = v;
-        v = pSrcPixels[3];
-
-        if (x < width/4-1) {
-            u1 = pSrcPixels[6];
-            v1 = pSrcPixels[9];
-        } else {
-            u1 = u;
-            v1 = v;
-        }
-
-        YUVtoBGR32Pixel(pDestPixel, pSrcPixels[1], u, v0/2+v/2);
-        YUVtoBGR32Pixel(pDestPixel+1, pSrcPixels[2], (u*3)/4+u1/4, v0/4+(v*3)/4);
-        YUVtoBGR32Pixel(pDestPixel+2, pSrcPixels[4], u/2+u1/2, v);
-        YUVtoBGR32Pixel(pDestPixel+3, pSrcPixels[5], u/4+(u1*3)/4, (v*3)/4+v1/4);
-
-        pSrcPixels+=6;
-        pDestPixel+=4;
-    }
-
-}
-
-void Camera::YUV411toBGR32(unsigned char* pSrc, BitmapPtr pBmp)
-{
-    Pixel32 * pBits = (Pixel32 *)(pBmp->getPixels());
-    for (int y = 0; y < getMediaHeight(); y++) {
-        Pixel32 * pDest = pBits+(y*pBmp->getStride())/4;
-        YUV411toBGR32Line(pSrc, y, pDest);
-    }
-}
-
-void Camera::YUV422toBGR32Line(unsigned char* pSrc, int y, Pixel32 * pDestLine)
-{
-    Pixel32 * pDestPixel = pDestLine;
-    int width = getMediaWidth();
-    // We need the previous and next values to interpolate between the
-    // sampled u and v values.
-    unsigned char v = *(pSrc+y*(width*2)+2);
-    unsigned char v0; // Previous v
-    unsigned char v1; // Next v;
-    unsigned char u;
-    unsigned char u1; // Next u;
-    unsigned char * pSrcPixels = pSrc+y*(width*2);
-
-    for (int x = 0; x < width/2; x++) {
-        // Two pixels at a time.
-        // Source format is UYVY.
-        u = pSrcPixels[0];
-        v0 = v;
-        v = pSrcPixels[2];
-
-        if (x < width/2-1) {
-            u1 = pSrcPixels[4];
-            v1 = pSrcPixels[6];
-        } else {
-            u1 = u;
-            v1 = v;
-        }
-
-        YUVtoBGR32Pixel(pDestPixel, pSrcPixels[1], u, v0/2+v/2);
-        YUVtoBGR32Pixel(pDestPixel+1, pSrcPixels[3], u/2+u1/2, v);
-
-        pSrcPixels+=4;
-        pDestPixel+=2;
-    }
-
-}
-
-void Camera::YUV422toBGR32(unsigned char* pSrc, BitmapPtr pBmp)
-{
-    Pixel32 * pBits = (Pixel32 *)(pBmp->getPixels());
-    for (int y = 0; y < getMediaHeight(); y++) {
-        Pixel32 * pDest = pBits+(y*pBmp->getStride())/4;
-        YUV422toBGR32Line(pSrc, y, pDest);
-    }
-}
-
 void Camera::dumpCameraInfo()
 {
     dc1394_camerainfo info;
