@@ -1,4 +1,4 @@
-//
+
 //  libavg - Media Playback Engine. 
 //  Copyright (C) 2003-2006 Ulrich von Zadow
 //
@@ -22,6 +22,9 @@
 #ifndef _DisplayEngine_H_
 #define _DisplayEngine_H_
 
+#include "Node.h"
+#include "DisplayParams.h"
+
 #include "../graphics/Rect.h"
 #include "../graphics/Pixel32.h"
 #include "../graphics/Bitmap.h"
@@ -33,8 +36,6 @@ namespace avg {
 class ISurface;
 
 class Region;
-class Node;
-class AVGNode;
 
 class DisplayEngine
 {	
@@ -43,8 +44,7 @@ class DisplayEngine
 
         DisplayEngine();
         virtual ~DisplayEngine();
-        virtual void init(int width, int height, bool isFullscreen, 
-                int bpp, int WindowWidth, int WindowHeight) = 0;
+        virtual void init(const DisplayParams& DP) = 0;
         virtual void teardown() = 0;
         void initRender();
         void deinitRender();
@@ -56,7 +56,7 @@ class DisplayEngine
         virtual void setGamma(double Red, double Green, double Blue) = 0;
 
 
-        virtual void render(AVGNode * pRootNode, bool bRenderEverything) = 0;
+        virtual void render(AVGNodePtr pRootNode, bool bRenderEverything) = 0;
         void frameWait();
         
         virtual void setClipRect() = 0;
