@@ -21,8 +21,10 @@
 
 #include "DivNode.h"
 #include "DisplayEngine.h"
+#include "Player.h"
 
 #include "../base/Exception.h"
+#include "../base/Logger.h"
 
 #include <iostream>
 #include <sstream>
@@ -35,24 +37,31 @@ DivNode::DivNode()
 {
 }
 
-DivNode::DivNode (const xmlNodePtr xmlNode, DivNode * pParent)
-    : Node(xmlNode, pParent)
+DivNode::DivNode (const xmlNodePtr xmlNode, Player * pPlayer)
+    : Node(xmlNode, pPlayer)
 {
     
 }
 
 DivNode::~DivNode()
 {
-    for (unsigned int i = 0; i< m_Children.size(); i++) {
-        delete m_Children[i];
-    }
 }
 
-void DivNode::init(DisplayEngine * pEngine, DivNode * pParent, 
-        Player * pPlayer)
+void DivNode::setDisplayEngine(DisplayEngine * pEngine)
 {
-    Node::init(pEngine, pParent, pPlayer);
-    Node::initVisible();
+    Node::setDisplayEngine(pEngine);
+    for  (int i = 0; i< (int)m_Children.size(); ++i) {
+        m_Children[i]->setDisplayEngine(pEngine);
+    }
+    
+}
+
+void DivNode::disconnect()
+{
+    for  (int i = 0; i< (int)m_Children.size(); ++i) {
+        m_Children[i]->disconnect();
+    }
+    Node::disconnect();
 }
 
 int DivNode::getNumChildren ()
@@ -60,30 +69,44 @@ int DivNode::getNumChildren ()
     return m_Children.size();
 }
 
-Node * DivNode::getChild (int i)
+NodePtr DivNode::getChild (int i)
 {
     if (i >= (int)m_Children.size() || i < 0) {
         stringstream s;
         s << "Index " << i << " is out of range in DivNode::getChild()";
         throw(Exception(AVG_ERR_OUT_OF_RANGE, s.str()));
-        
     }
     return m_Children[i];
 }
 
-void DivNode::addChild (Node * pNewNode)
+void DivNode::addChild (NodePtr pNewNode)
 {
+    if (pNewNode->getState() == NS_CONNECTED) {
+        throw(Exception(AVG_ERR_ALREADY_CONNECTED,
+                "Can't connect node with id "+pNewNode->getID()+
+                ": already connected."));
+    }
     m_Children.push_back(pNewNode);
+    DivNodePtr Ptr = boost::dynamic_pointer_cast<DivNode>(getThis());
+    pNewNode->setParent(Ptr);
+    if (getState() == NS_CONNECTED) {
+        getPlayer()->addNodeID(pNewNode);
+    }
+    if (isDisplayAvailable()) {
+        pNewNode->setDisplayEngine(getEngine());
+    }
 }
 
 void DivNode::removeChild (int i)
 {
-    Node * pNode = getChild(i);
+    NodePtr pNode = getChild(i);
     pNode->invalidate();
+    pNode->setParent(DivNodePtr());
+    pNode->disconnect();
     m_Children.erase(m_Children.begin()+i);
 }
 
-int DivNode::indexOf(Node * pChild)
+int DivNode::indexOf(NodePtr pChild)
 {
     for  (int i = 0; i< (int)m_Children.size(); ++i) {
         if (m_Children[i] == pChild) {
@@ -93,18 +116,18 @@ int DivNode::indexOf(Node * pChild)
     return -1;
 }
 
-Node * DivNode::getElementByPos (const DPoint & pos)
+NodePtr DivNode::getElementByPos (const DPoint & pos)
 {
     if (!getVisibleRect().Contains(pos) || !reactsToMouseEvents()) {
-        return 0;
+        return NodePtr();
     }
     for (int i=getNumChildren()-1; i>=0; i--) {
-        Node * pFoundNode = getChild(i)->getElementByPos(pos);
+        NodePtr pFoundNode = getChild(i)->getElementByPos(pos);
         if (pFoundNode) {
             return pFoundNode;
         }
     }
-    return this; // pos is in current node, but not in any child.
+    return getThis(); // pos is in current node, but not in any child.
 }
 
 void DivNode::prepareRender (int time, const DRect& parent)
@@ -154,7 +177,7 @@ string DivNode::getTypeStr ()
 string DivNode::dump (int indent)
 {
     string dumpStr = Node::dump () + "\n";
-    vector<Node*>::iterator it;
+    vector<NodePtr>::iterator it;
     for (it=m_Children.begin(); it<m_Children.end(); it++) {
         dumpStr += (*it)->dump(indent+2)+"\n";
     }
