@@ -34,18 +34,18 @@ class DivNode : public Node
 {
 	public:
         DivNode ();
-        DivNode (const xmlNodePtr xmlNode, DivNode * pParent);
+        DivNode (const xmlNodePtr xmlNode, Player * pPlayer);
         virtual ~DivNode ();
-        virtual void init(DisplayEngine * pEngine, DivNode * pParent, 
-                Player * pPlayer);
+        virtual void setDisplayEngine(DisplayEngine * pEngine);
+        virtual void disconnect();
 
         int getNumChildren ();
-        Node * getChild (int i);
-        void addChild (Node * newNode);
+        NodePtr getChild (int i);
+        void addChild (NodePtr newNode);
         void removeChild (int i);
-        int indexOf(Node * pChild);
+        int indexOf(NodePtr pChild);
 
-        virtual Node * getElementByPos (const DPoint & pos);
+        virtual NodePtr getElementByPos (const DPoint & pos);
         virtual void prepareRender (int time, const DRect& parent);
         virtual void render (const DRect& rect);
         virtual bool obscures (const DRect& rect, int Child);
@@ -58,7 +58,7 @@ class DivNode : public Node
         virtual Point<double> getPreferredMediaSize();	
     
     private:
-        std::vector <Node *> m_Children;
+        std::vector <NodePtr> m_Children;
 };
 
 }
