@@ -22,13 +22,9 @@
 #ifndef _OGLShader_H_
 #define _OGLShader_H_
 
-#include <../base/CountedPointer.h>
+#include <boost/shared_ptr.hpp>
 
-#include "GL/gl.h"
-#ifndef __APPLE__
-#define GLX_GLXEXT_PROTOTYPES
-#include "GL/glx.h"
-#endif
+#include "OGLHelper.h"
 
 #include <string>
 
@@ -49,7 +45,7 @@ class OGLShader {
         std::string m_sProgram;
 };
 
-typedef CountedPointer<OGLShader> OGLShaderPtr;
+typedef boost::shared_ptr<OGLShader> OGLShaderPtr;
 
 }
 
