@@ -20,6 +20,7 @@
 //
 
 #include "OGLSurface.h"
+#include "OGLHelper.h"
 #include "Player.h"
 #include "MathHelper.h"
 #include "../base/Logger.h"
@@ -27,8 +28,6 @@
 #include "../base/ScopeTimer.h"
 #include "../graphics/Point.h"
 
-#include "GL/glu.h"
-
 #include <iostream>
 #include <sstream>
 
@@ -36,12 +35,6 @@ using namespace std;
 
 namespace avg {
 
-/*
-#ifndef __APPLE__
-PFNGLXALLOCATEMEMORYMESAPROC OGLSurface::s_AllocMemMESAProc = 0;
-PFNGLXFREEMEMORYMESAPROC OGLSurface::s_FreeMemMESAProc = 0;
-#endif
-*/
 OGLSurface::OGLSurface(SDLDisplayEngine * pEngine)
     : m_pEngine(pEngine),
       m_bBound(false),
@@ -59,7 +52,7 @@ OGLSurface::~OGLSurface()
     unbind();
     switch(m_MemoryMode) {
         case PBO:
-            if (m_pf == YCbCr420p) {
+            if (m_pf == YCbCr420p || m_pf == YCbCrJ420p) {
                 for (int i=0; i<3; i++) {
                     glproc::DeleteBuffers(1, &m_hPixelBuffers[i]);
                 }
@@ -88,7 +81,7 @@ void OGLSurface::create(const IntPoint& Size, PixelFormat pf, bool bFastDownload
     if (bFastDownload) {
         m_MemoryMode = m_pEngine->getMemoryModeSupported();
     }
-    if (m_pf == YCbCr420p) {
+    if (m_pf == YCbCr420p || m_pf == YCbCrJ420p) {
         createBitmap(Size, I8, 0);
         IntPoint HalfSize(Size.x/2, Size.y/2);
         createBitmap(HalfSize, I8, 1);
@@ -137,7 +130,7 @@ BitmapPtr OGLSurface::lockBmp(int i)
                     Size = IntPoint(m_Size.x/2, m_Size.y/2);
                 }
                 PixelFormat pf;
-                if (m_pf == YCbCr420p) {
+                if (m_pf == YCbCr420p || m_pf == YCbCrJ420p) {
                     pf = I8;
                 } else {
                     pf = m_pf;
@@ -146,16 +139,6 @@ BitmapPtr OGLSurface::lockBmp(int i)
                         Size.x*Bitmap::getBytesPerPixel(pf), false));
             }
             break;
-/*
-#ifndef __APPLE__
-        case MESA:
-            {
-                int stride = m_Size.x*Bitmap::getBytesPerPixel(m_pf);
-                m_pBmp = BitmapPtr(new Bitmap(m_Size, m_pf, 
-                        (unsigned char *)m_pMESABuffer, stride, false));
-            }
-#endif
-*/
         default:
             break;
     }
@@ -165,7 +148,7 @@ BitmapPtr OGLSurface::lockBmp(int i)
 void OGLSurface::unlockBmps()
 {
 //    cerr << "unlockBmps" << endl;
-    if (m_pf == YCbCr420p) {
+    if (m_pf == YCbCr420p || m_pf == YCbCrJ420p) {
         for (int i=0; i<3; i++) {
             unlockBmp(i);
         }
@@ -301,7 +284,7 @@ void OGLSurface::bind()
                         m_Size.x, m_pf, m_pEngine));
                 m_pTiles[y].push_back(pTile);
                 if (m_MemoryMode == PBO) {
-                    if (m_pf == YCbCr420p) {
+                    if (m_pf == YCbCr420p || m_pf == YCbCrJ420p) {
                         for (int i=0; i<3; i++) {
                             glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, 
                                     m_hPixelBuffers[i]);
@@ -349,7 +332,7 @@ void OGLSurface::rebind()
         for (unsigned int x=0; x<m_pTiles[y].size(); x++) {
             OGLTilePtr pTile = m_pTiles[y][x];
             if (m_MemoryMode == PBO) {
-                if (m_pf == YCbCr420p) {
+                if (m_pf == YCbCr420p || m_pf == YCbCrJ420p) {
                     for (int i=0; i<3; i++) {
                         glproc::BindBuffer(GL_PIXEL_UNPACK_BUFFER_EXT, m_hPixelBuffers[i]);
                         OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
@@ -463,23 +446,6 @@ void OGLSurface::createBitmap(const IntPoint& Size, PixelFormat pf, int i)
                     "OGLSurface::createBitmap: glBindBuffer(0)");
             m_pBmps[i] = BitmapPtr();
             break;
-/*            
-#ifndef __APPLE__
-        case MESA:
-            {
-                Display * display = XOpenDisplay(0);
-                int stride = m_Size.x*Bitmap::getBytesPerPixel(m_pf);
-                m_pMESABuffer = s_AllocMemMESA(display, DefaultScreen(display),
-                        stride*(Size.y+1), 0, 1.0 ,0);
-                if (!m_pMESABuffer) {
-                    AVG_TRACE(Logger::WARNING, "Failed to allocate MESA memory");
-                    m_MemoryMode = OGL;
-                }
-                m_pBmps[index] = BitmapPtr();
-            }
-            break;
-#endif
-*/
         default:
             break;
     }
@@ -529,22 +495,22 @@ void OGLSurface::bltTexture(const DRect* pDestRect,
 
     switch(Mode) {
         case DisplayEngine::BLEND_BLEND:
-            glBlendEquation(GL_FUNC_ADD);
+            glproc::BlendEquation(GL_FUNC_ADD);
             glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
             checkBlendModeError("blend");
             break;
         case DisplayEngine::BLEND_ADD:
-            glBlendEquation(GL_FUNC_ADD);
+            glproc::BlendEquation(GL_FUNC_ADD);
             glBlendFunc(GL_SRC_ALPHA, GL_ONE);
             checkBlendModeError("add");
             break;
         case DisplayEngine::BLEND_MIN:
-            glBlendEquation(GL_MIN);
+            glproc::BlendEquation(GL_MIN);
             glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
             checkBlendModeError("min");
             break;
         case DisplayEngine::BLEND_MAX:
-            glBlendEquation(GL_MAX);
+            glproc::BlendEquation(GL_MAX);
             glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
             checkBlendModeError("max");
             break;
