@@ -62,6 +62,11 @@
 
 #include <libxml/xmlmemory.h>
 
+#ifdef _WIN32
+#include <direct.h>
+#define getcwd _getcwd
+#endif
+
 #include <algorithm>
 #include <iostream>
 #include <sstream>
@@ -72,16 +77,24 @@ using namespace std;
 namespace avg {
 
 Player::Player()
-    : m_pRootNode (0),
+    : m_pRootNode(),
       m_pDisplayEngine(0),
       m_TestHelper(this),
       m_bInHandleTimers(false),
-      m_pLastMouseNode(0),
-      m_bShowCursor(true),
+      m_bCurrentTimeoutDeleted(false), 
+      m_pLastMouseNode(),
+      m_pEventCaptureNode(),
       m_bIsPlaying(false)
 {
     initConfig();
+    // Find and parse dtd.
     registerDTDEntityLoader(g_pAVGDTD);
+    string sDTDFName = "avg.dtd";
+    m_dtd = xmlParseDTD(NULL, (const xmlChar*) sDTDFName.c_str());
+    if (!m_dtd) {
+        AVG_TRACE(Logger::WARNING, 
+                "DTD not found at " << sDTDFName << ". Not validating xml files.");
+    }
 }
 
 Player::~Player()
@@ -114,33 +127,21 @@ void Player::setDisplayEngine(DisplayEngineType engine)
 void Player::setResolution(bool bFullscreen, 
                 int width, int height, int bpp)
 {
-    if (m_pRootNode) {
-        AVG_TRACE(Logger::ERROR,
-                "Player::setResolution called after loadFile."
-                << " Aborting.");
-        exit(-1);
-    }
-    m_bFullscreen = bFullscreen;
+    m_DP.m_bFullscreen = bFullscreen;
     if (bpp) {
-        m_BPP = bpp;
+        m_DP.m_BPP = bpp;
     }
     if (width) {
-        m_WindowWidth = width;
+        m_DP.m_WindowWidth = width;
     }
     if (height) {
-        m_WindowHeight = height;
+        m_DP.m_WindowHeight = height;
     }
 }
         
 void Player::setOGLOptions(bool bUsePOW2Textures, DisplayEngine::YCbCrMode DesiredYCbCrMode, 
                 bool bUseRGBOrder, bool bUsePixelBuffers, int MultiSampleSamples)
 {
-    if (m_pRootNode) {
-        AVG_TRACE(Logger::ERROR,
-                "Player::setOGLOptions called before loadFile."
-                << " Aborting.");
-        exit(-1);
-    }
     m_bUsePOW2Textures = bUsePOW2Textures;
     m_YCbCrMode = DesiredYCbCrMode;
     m_bUseRGBOrder = bUseRGBOrder;
@@ -153,87 +154,18 @@ void Player::loadFile (const std::string& filename)
     try {
         AVG_TRACE(Logger::PROFILE, 
                 std::string("Player::LoadFile(") + filename + ")");
+        if (m_pRootNode) {
+            cleanup();
+        }
         assert (!m_pRootNode);
 
-        // Init display configuration.
-        AVG_TRACE(Logger::CONFIG, "Display subsystem: " << 
-                m_sDisplaySubsystem);
-        AVG_TRACE(Logger::CONFIG, "Display bpp: " << m_BPP);
-        AVG_TRACE(Logger::CONFIG, "Display fullscreen: "
-                << (m_bFullscreen?"true":"false"));
-
-        if (!m_pDisplayEngine) {
-            if (m_sDisplaySubsystem == "DFB") {
-#ifdef AVG_ENABLE_DFB
-                m_pDisplayEngine = new DFBDisplayEngine ();
-                m_pEventSource =
-                    dynamic_cast<DFBDisplayEngine *>(m_pDisplayEngine);
-#else
-                AVG_TRACE(Logger::ERROR,
-                        "Display subsystem set to DFB but no DFB support compiled."
-                        << " Aborting.");
-                exit(-1);
-#endif
-            } else if (m_sDisplaySubsystem == "OGL") {
-#ifdef AVG_ENABLE_GL
-                AVG_TRACE(Logger::CONFIG, "Requested OGL configuration: ");
-                AVG_TRACE(Logger::CONFIG, "  POW2 textures: " 
-                        << (m_bUsePOW2Textures?"true":"false"));
-                string sMode;
-                switch (m_YCbCrMode) {
-                    case DisplayEngine::NONE:
-                        AVG_TRACE(Logger::CONFIG, "  No YCbCr texture support.");
-                        break;
-                    case DisplayEngine::OGL_MESA:
-                        AVG_TRACE(Logger::CONFIG, "  Mesa YCbCr texture support.");
-                        break;
-                    case DisplayEngine::OGL_APPLE:
-                        AVG_TRACE(Logger::CONFIG, "  Apple YCbCr texture support.");
-                        break;
-                    case DisplayEngine::OGL_SHADER:
-                        AVG_TRACE(Logger::CONFIG, "  Fragment shader YCbCr texture support.");
-                        break;
-                }
-                AVG_TRACE(Logger::CONFIG, "  RGB order: " 
-                        << (m_bUseRGBOrder?"true":"false"));
-                AVG_TRACE(Logger::CONFIG, "  Use pixel buffers: " 
-                        << (m_bUsePixelBuffers?"true":"false"));
-                AVG_TRACE(Logger::CONFIG, "  Multisample samples: " 
-                        << m_MultiSampleSamples);
-                
-                m_pDisplayEngine = new SDLDisplayEngine ();
-                m_pEventSource = 
-                    dynamic_cast<SDLDisplayEngine *>(m_pDisplayEngine);
-#else
-                AVG_TRACE(Logger::ERROR,
-                        "Display subsystem set to GL but no GL support compiled."
-                        << " Aborting.");
-                exit(-1);
-#endif
-            } else {
-                AVG_TRACE(Logger::ERROR, 
-                        "Display subsystem set to unknown value " << 
-                        m_sDisplaySubsystem << ". Aborting.");
-                exit(-1);
-            }
-        }
-
-        // Find and parse dtd.
-        // PREFIXDIR is the install prefix set by configure.
-
-        string sDTDFName = "avg.dtd";
-        xmlDtdPtr dtd = xmlParseDTD(NULL, (const xmlChar*) sDTDFName.c_str());
-        if (!dtd) {
-            AVG_TRACE(Logger::WARNING, 
-                    "DTD not found at " << sDTDFName << ". Not validating xml files.");
-        }
-
-        // Construct path.
+        // When loading an avg file, assets are loaded from a directory relative
+        // to the file.
+        char szBuf[1024];
         string RealFilename;
         if (filename[0] == '/') {
             RealFilename = filename; 
         } else {
-            char szBuf[1024];
             getcwd(szBuf, 1024);
             m_CurDirName = string(szBuf)+"/";
             RealFilename = m_CurDirName+filename;
@@ -253,24 +185,31 @@ void Player::loadFile (const std::string& filename)
         xmlValidCtxtPtr cvp = xmlNewValidCtxt();
         cvp->error = xmlParserValidityError;
         cvp->warning = xmlParserValidityWarning;
-        int valid=xmlValidateDtd(cvp, doc, dtd);  
+        int valid=xmlValidateDtd(cvp, doc, m_dtd);  
         xmlFreeValidCtxt(cvp);
         if (!valid) {
             throw (Exception(AVG_ERR_XML_PARSE, 
                     filename + " does not validate."));
         }
+        xmlNodePtr xmlNode = xmlDocGetRootElement(doc);
+        m_pRootNode = boost::dynamic_pointer_cast<AVGNode>
+            (createNodeFromXml(doc, xmlNode, DivNodePtr()));
+        m_pRootNode->setParent(DivNodeWeakPtr());
+        m_DP.m_Height = getDefaultedIntAttr (xmlNode, "height", 0);
+        m_DP.m_Width = getDefaultedIntAttr (xmlNode, "width", 0);
+        registerNode(m_pRootNode);
 
-        m_pRootNode = dynamic_cast<AVGNode*>
-            (createNodeFromXml(doc, xmlDocGetRootElement(doc), 0));
-        initDisplay(xmlDocGetRootElement(doc));
-        initNode(m_pRootNode, 0);
-        DRect rc = m_pRootNode->getRelViewport();
-
+        // Reset the directory to load assets from to the current dir.
+        getcwd(szBuf, 1024);
+        m_CurDirName = string(szBuf)+"/";
+        
         xmlFreeDoc(doc);
     } catch (Exception& ex) {
         AVG_TRACE(Logger::ERROR, ex.GetStr());
+        throw;
     } catch (Magick::Exception & ex) {
         AVG_TRACE(Logger::ERROR, ex.what());
+        throw;
     }
 }
 
@@ -282,17 +221,23 @@ void Player::play()
             AVG_TRACE(Logger::ERROR, "play called, but no xml file loaded.");
         }
         assert(m_pRootNode);
+        initGraphics();
+        m_pRootNode->setDisplayEngine(m_pDisplayEngine);
         
         m_EventDispatcher.addSource(m_pEventSource);
+        m_EventDispatcher.addSource(&m_TestHelper);
         m_EventDispatcher.addSink(&m_EventDumper);
         m_EventDispatcher.addSink(this);
         
         m_pDisplayEngine->initRender();
         m_bStopping = false;
 
-        m_pDisplayEngine->render(m_pRootNode, true);
-        
         Profiler::get().start();
+        m_pDisplayEngine->render(m_pRootNode, true);
+        if (m_pDisplayEngine->wasFrameLate()) {
+            Profiler::get().dumpFrame();
+        }
+        
         try {
             while (!m_bStopping) {
                 doFrame();
@@ -304,15 +249,20 @@ void Player::play()
         }
         cleanup();
 
-    } catch  (Exception& ex) {
+    } catch (Exception& ex) {
         AVG_TRACE(Logger::ERROR, ex.GetStr());
+        throw;
     }
     m_bIsPlaying = false;
 }
 
 void Player::stop ()
 {
-    m_bStopping = true;
+    if (m_bIsPlaying) {
+        m_bStopping = true;
+    } else {
+        cleanup();
+    }
 }
 
 bool Player::isPlaying()
@@ -321,21 +271,23 @@ bool Player::isPlaying()
 }
 
 void Player::setFramerate(double rate) {
-    if (!m_pDisplayEngine) {
-        AVG_TRACE(Logger::ERROR, 
-                "Player::setFramerate called without a loaded avg file. Aborting.");
-        exit(-1);
+    if (m_bIsPlaying) {
+        m_pDisplayEngine->setFramerate(rate);
+    } else {
+        m_DP.m_Framerate = rate;
+        m_DP.m_VBRate = 0;
     }
-    m_pDisplayEngine->setFramerate(rate);
 }
 
 bool Player::setVBlankFramerate(int rate) {
-    if (!m_pDisplayEngine) {
-        AVG_TRACE(Logger::ERROR, 
-                "Player::setVBlankFramerate called without a loaded avg file. Aborting.");
-        exit(-1);
+    // TODO: Why does this function return anything?
+    if (m_bIsPlaying) {
+        return m_pDisplayEngine->setVBlankRate(rate);
+    } else {
+        m_DP.m_Framerate = 0;
+        m_DP.m_VBRate = rate;
+        return true;
     }
-    return m_pDisplayEngine->setVBlankRate(rate);
 }
 
 TestHelper * Player::getTestHelper()
@@ -370,13 +322,11 @@ bool Player::clearInterval(int id)
     vector<Timeout*>::iterator it;
     for (it=m_PendingTimeouts.begin(); it!=m_PendingTimeouts.end(); it++) {
         if (id == (*it)->GetID()) {
-            if (m_bInHandleTimers) {
-                // Can't kill timeouts during timeout handling...
-                m_KilledTimeouts.push_back(id);
-            } else {
-                delete *it;
-                m_PendingTimeouts.erase(it);
+            if (it == m_PendingTimeouts.begin()) {
+                m_bCurrentTimeoutDeleted = true;
             }
+            delete *it;
+            m_PendingTimeouts.erase(it);
             return true;
         }
     }
@@ -404,22 +354,68 @@ void Player::showCursor(bool bShow)
     if (m_pDisplayEngine) {
         m_pDisplayEngine->showCursor(bShow);
     } else {
-        m_bShowCursor = bShow;
+        m_DP.m_bShowCursor = bShow;
     }
 }
 
+void Player::setEventCapture(NodeWeakPtr pNode) {
+    if (!m_pEventCaptureNode.expired()) {
+        throw Exception(AVG_ERR_INVALID_CAPTURE,
+                "setEventCapture called, but mouse already captured.");
+    } else {
+        m_pEventCaptureNode = pNode;
+    }
+}
 
-Node * Player::getElementByID (const std::string& id)
+void Player::releaseEventCapture(NodeWeakPtr pNode) {
+    if (m_pEventCaptureNode.expired()) {
+        throw Exception(AVG_ERR_INVALID_CAPTURE,
+                "releaseEventCapture called, but mouse not captured.");
+    } else {
+        m_pEventCaptureNode = NodeWeakPtr();
+    }
+
+}
+
+NodePtr Player::getElementByID (const std::string& id)
 {
     if (m_IDMap.find(id) != m_IDMap.end()) {
         return m_IDMap.find(id)->second;
     } else {
-        AVG_TRACE(Logger::WARNING, "getElementByID(" << id << ") failed.");
-        return 0;
+        AVG_TRACE(Logger::WARNING, "getElementByID(\"" << id << "\") failed.");
+        return NodePtr();
+    }
+}
+        
+void Player::addNodeID(NodePtr pNode)
+{
+    const string& id = pNode->getID();
+    if (id != "") {
+        if (m_IDMap.find(id) != m_IDMap.end() &&
+            m_IDMap.find(id)->second != pNode)
+        {
+            throw (Exception (AVG_ERR_XML_DUPLICATE_ID,
+                string("Error: duplicate id ")+id));
+        }
+        m_IDMap.insert(NodeIDMap::value_type(id, pNode));
     }
 }
 
-AVGNode * Player::getRootNode ()
+void Player::removeNodeID(const std::string& id)
+{
+    if (id != "") {
+        std::map<std::string, NodePtr>::iterator it;
+        it = m_IDMap.find(id);
+        if (it != m_IDMap.end()) {
+            m_IDMap.erase(it);
+        } else {
+            AVG_TRACE(Logger::ERROR, "removeNodeID("+id+") failed.");
+            exit(1);
+        }
+    }
+}
+
+AVGNodePtr Player::getRootNode ()
 {
     return m_pRootNode;
 }
@@ -493,7 +489,7 @@ void Player::doFrame ()
 double Player::getFramerate ()
 {
     if (!m_pDisplayEngine) {
-        return 0;
+        return m_DP.m_Framerate;
     }
     return m_pDisplayEngine->getFramerate();
 }
@@ -508,11 +504,13 @@ double Player::getVideoRefreshRate()
 
 void Player::setGamma(double Red, double Green, double Blue)
 {
-    if (!m_pDisplayEngine) {
-        AVG_TRACE(Logger::ERROR, 
-                "Player::setVBlankFramerate called without a loaded avg file. Ignoring.");
+    if (m_pDisplayEngine) {
+        m_pDisplayEngine->setGamma(Red, Green, Blue);
+    } else {
+        m_DP.m_Gamma[0] = Red;
+        m_DP.m_Gamma[1] = Green;
+        m_DP.m_Gamma[2] = Blue;
     }
-    return m_pDisplayEngine->setGamma(Red, Green, Blue);
 }
 
 void Player::initConfig() {
@@ -520,25 +518,25 @@ void Player::initConfig() {
     ConfigMgr* pMgr = ConfigMgr::get();
     
     m_sDisplaySubsystem = *pMgr->getOption("scr", "subsys");
-    
-    m_BPP = atoi(pMgr->getOption("scr", "bpp")->c_str());
-    if (m_BPP != 15 && m_BPP != 16 && m_BPP != 24 && m_BPP != 32) {
+   
+    m_DP.m_BPP = atoi(pMgr->getOption("scr", "bpp")->c_str());
+    if (m_DP.m_BPP != 15 && m_DP.m_BPP != 16 && m_DP.m_BPP != 24 && m_DP.m_BPP != 32) {
         AVG_TRACE(Logger::ERROR, 
                 "BPP must be 15, 16, 24 or 32. Current value is " 
-                << m_BPP << ". Aborting." );
+                << m_DP.m_BPP << ". Aborting." );
         exit(-1);
     }
-    m_bFullscreen = pMgr->getBoolOption("scr", "fullscreen", false);
+    m_DP.m_bFullscreen = pMgr->getBoolOption("scr", "fullscreen", false);
 
-    m_WindowWidth = atoi(pMgr->getOption("scr", "windowwidth")->c_str());
-    m_WindowHeight = atoi(pMgr->getOption("scr", "windowheight")->c_str());
+    m_DP.m_WindowWidth = atoi(pMgr->getOption("scr", "windowwidth")->c_str());
+    m_DP.m_WindowHeight = atoi(pMgr->getOption("scr", "windowheight")->c_str());
 
-    if (m_bFullscreen && (m_WindowWidth != 0 || m_WindowHeight != 0)) {
+    if (m_DP.m_bFullscreen && (m_DP.m_WindowWidth != 0 || m_DP.m_WindowHeight != 0)) {
         AVG_TRACE(Logger::ERROR, 
                 "Can't set fullscreen and window size at once. Aborting.");
         exit(-1);
     }
-    if (m_WindowWidth != 0 && m_WindowHeight != 0) {
+    if (m_DP.m_WindowWidth != 0 && m_DP.m_WindowHeight != 0) {
         AVG_TRACE(Logger::ERROR, "Can't set window width and height at once");
         AVG_TRACE(Logger::ERROR, 
                 "(aspect ratio is determined by avg file). Aborting.");
@@ -576,74 +574,169 @@ void Player::initConfig() {
         m_bUseRGBOrder = pMgr->getBoolOption("scr", "usergborder", false);
         m_bUsePixelBuffers = pMgr->getBoolOption("scr", "usepixelbuffers", true);
         m_MultiSampleSamples = pMgr->getIntOption("scr", "multisamplesamples", 1);
-        m_Gamma[0] = -1.0;
-        m_Gamma[1] = -1.0;
-        m_Gamma[2] = -1.0;
-        pMgr->getGammaOption("scr", "gamma", m_Gamma);
+        pMgr->getGammaOption("scr", "gamma", m_DP.m_Gamma);
     }
 }
 
-Node * Player::createNodeFromXmlString (const string& sXML)
+void Player::initGraphics()
+{
+    // Init display configuration.
+    AVG_TRACE(Logger::CONFIG, "Display subsystem: " << 
+            m_sDisplaySubsystem);
+    AVG_TRACE(Logger::CONFIG, "Display bpp: " << m_DP.m_BPP);
+    AVG_TRACE(Logger::CONFIG, "Display fullscreen: "
+            << (m_DP.m_bFullscreen?"true":"false"));
+
+    if (!m_pDisplayEngine) {
+        if (m_sDisplaySubsystem == "DFB") {
+#ifdef AVG_ENABLE_DFB
+            m_pDisplayEngine = new DFBDisplayEngine ();
+            m_pEventSource =
+                dynamic_cast<DFBDisplayEngine *>(m_pDisplayEngine);
+#else
+            AVG_TRACE(Logger::ERROR,
+                    "Display subsystem set to DFB but no DFB support compiled."
+                    << " Aborting.");
+            exit(-1);
+#endif
+        } else if (m_sDisplaySubsystem == "OGL") {
+#ifdef AVG_ENABLE_GL
+            AVG_TRACE(Logger::CONFIG, "Requested OGL configuration: ");
+            AVG_TRACE(Logger::CONFIG, "  POW2 textures: " 
+                    << (m_bUsePOW2Textures?"true":"false"));
+            string sMode;
+            switch (m_YCbCrMode) {
+                case DisplayEngine::NONE:
+                    AVG_TRACE(Logger::CONFIG, "  No YCbCr texture support.");
+                    break;
+                case DisplayEngine::OGL_MESA:
+                    AVG_TRACE(Logger::CONFIG, "  Mesa YCbCr texture support.");
+                    break;
+                case DisplayEngine::OGL_APPLE:
+                    AVG_TRACE(Logger::CONFIG, "  Apple YCbCr texture support.");
+                    break;
+                case DisplayEngine::OGL_SHADER:
+                    AVG_TRACE(Logger::CONFIG, "  Fragment shader YCbCr texture support.");
+                    break;
+            }
+            AVG_TRACE(Logger::CONFIG, "  RGB order: " 
+                    << (m_bUseRGBOrder?"true":"false"));
+            AVG_TRACE(Logger::CONFIG, "  Use pixel buffers: " 
+                    << (m_bUsePixelBuffers?"true":"false"));
+            AVG_TRACE(Logger::CONFIG, "  Multisample samples: " 
+                    << m_MultiSampleSamples);
+
+            m_pDisplayEngine = new SDLDisplayEngine ();
+            m_pEventSource = 
+                dynamic_cast<SDLDisplayEngine *>(m_pDisplayEngine);
+#else
+            AVG_TRACE(Logger::ERROR,
+                    "Display subsystem set to GL but no GL support compiled."
+                    << " Aborting.");
+            exit(-1);
+#endif
+        } else {
+            AVG_TRACE(Logger::ERROR, 
+                    "Display subsystem set to unknown value " << 
+                    m_sDisplaySubsystem << ". Aborting.");
+            exit(-1);
+        }
+    }
+    SDLDisplayEngine * pSDLDisplayEngine = 
+            dynamic_cast<SDLDisplayEngine*>(m_pDisplayEngine);
+    if (pSDLDisplayEngine) {
+        pSDLDisplayEngine->setOGLOptions(m_bUsePOW2Textures, m_YCbCrMode, 
+                m_bUseRGBOrder, m_bUsePixelBuffers, m_MultiSampleSamples);
+    }
+    m_pDisplayEngine->init(m_DP);
+}
+
+void Player::registerNode(NodePtr pNode)
+{
+    addNodeID(pNode);
+    DivNodePtr pDivNode = boost::dynamic_pointer_cast<DivNode>(pNode);
+    if (pDivNode) {
+        for (int i=0; i<pDivNode->getNumChildren(); i++) {
+            registerNode(pDivNode->getChild(i));
+        }
+    }
+}
+
+NodePtr Player::createNodeFromXmlString (const string& sXML)
 {
     try {
+        xmlPedanticParserDefault(1);
+        xmlDoValidityCheckingDefaultValue =0;
+        
         xmlDocPtr doc;
         doc = xmlParseMemory(sXML.c_str(), sXML.length());
         if (!doc) {
             throw (Exception(AVG_ERR_XML_PARSE, 
                         string("Error parsing xml:\n  ")+sXML));
         }
-        Node * pNode = createNodeFromXml(doc, xmlDocGetRootElement(doc), 0);
+        NodePtr pNode = createNodeFromXml(doc, xmlDocGetRootElement(doc), DivNodePtr());
+
+        xmlValidCtxtPtr cvp = xmlNewValidCtxt();
+        cvp->error = xmlParserValidityError;
+        cvp->warning = xmlParserValidityWarning;
+        int valid=xmlValidateDtd(cvp, doc, m_dtd);  
+        xmlFreeValidCtxt(cvp);
+        if (!valid) {
+            throw (Exception(AVG_ERR_XML_PARSE, 
+                    "Could not validate '"+sXML+"'"));
+        }
 
         xmlFreeDoc(doc);
         return pNode;
     } catch (Exception& ex) {
         AVG_TRACE(Logger::ERROR, ex.GetStr());
-        return 0;
+        return NodePtr();
     } catch (Magick::Exception& ex) {
         AVG_TRACE(Logger::ERROR, ex.what());
-        return 0;
+        return NodePtr();
     }
 }
 
-Node * Player::createNodeFromXml (const xmlDocPtr xmlDoc, 
-        const xmlNodePtr xmlNode, DivNode * pParent)
+NodePtr Player::createNodeFromXml (const xmlDocPtr xmlDoc, 
+        const xmlNodePtr xmlNode, DivNodeWeakPtr pParent)
 {
     const char * nodeType = (const char *)xmlNode->name;
-    Node * curNode = 0;
+    NodePtr curNode;
     
     string id = getDefaultedStringAttr (xmlNode, "id", "");
     if (!strcmp (nodeType, "avg")) {
-        curNode = new AVGNode(xmlNode);
+        curNode = NodePtr(new AVGNode(xmlNode, this));
     } else if (!strcmp (nodeType, "div")) {
-        curNode = new DivNode(xmlNode, pParent);
+        curNode = NodePtr(new DivNode(xmlNode, this));
     } else if (!strcmp (nodeType, "image")) {
-        curNode = new Image(xmlNode, pParent);
+        curNode = NodePtr(new Image(xmlNode, this));
     } else if (!strcmp (nodeType, "words")) {
-        curNode = new Words(xmlNode, pParent);
+        curNode = NodePtr(new Words(xmlNode, this));
         string s = getXmlChildrenAsString(xmlDoc, xmlNode);
-        dynamic_cast<Words*>(curNode)->initText(s);
+        boost::dynamic_pointer_cast<Words>(curNode)->initText(s);
     } else if (!strcmp (nodeType, "video")) {
-        curNode = new Video(xmlNode, pParent);
+        curNode = NodePtr(new Video(xmlNode, this));
     } else if (!strcmp (nodeType, "camera")) {
-        curNode = new Camera(xmlNode, pParent);
+        curNode = NodePtr(new Camera(xmlNode, this));
     }
     else if (!strcmp (nodeType, "panoimage")) {
-        curNode = new PanoImage(xmlNode, pParent);
+        curNode = NodePtr(new PanoImage(xmlNode, this));
     } 
     else if (!strcmp (nodeType, "text") || 
                !strcmp (nodeType, "comment")) {
         // Ignore whitespace & comments
-        return 0;
+        return NodePtr();
     } else {
         throw (Exception (AVG_ERR_XML_NODE_UNKNOWN, 
             string("Unknown node type ")+(const char *)nodeType+" encountered."));
     }
+    curNode->setThis(curNode);
     // If this is a container, recurse into children
-    DivNode * curDivNode = dynamic_cast<DivNode*>(curNode);
+    DivNodePtr curDivNode = boost::dynamic_pointer_cast<DivNode>(curNode);
     if (curDivNode) {
         xmlNodePtr curXmlChild = xmlNode->xmlChildrenNode;
         while (curXmlChild) {
-            Node *curChild = createNodeFromXml (xmlDoc, curXmlChild, 
+            NodePtr curChild = createNodeFromXml (xmlDoc, curXmlChild, 
                     curDivNode);
             if (curChild) {
                 curDivNode->addChild(curChild);
@@ -654,76 +747,32 @@ Node * Player::createNodeFromXml (const xmlDocPtr xmlDoc,
     return curNode;
 }
 
-void Player::initNode(Node * pNode, DivNode * pParent)
-{
-    const string& ID = pNode->getID();
-    pNode->init(m_pDisplayEngine, pParent, this);
-    pNode->initVisible();
-    // If this is a container, recurse into children
-    DivNode * curDivNode = dynamic_cast<DivNode*>(pNode);
-    if (curDivNode) {
-        for (int i=0; i<curDivNode->getNumChildren(); ++i) {
-            initNode(curDivNode->getChild(i), curDivNode);
-        }
-    }
-    if (ID != "") {
-        if (m_IDMap.find(ID) != m_IDMap.end()) {
-            throw (Exception (AVG_ERR_XML_DUPLICATE_ID,
-                string("Error: duplicate id ")+ID));
-        }
-        m_IDMap.insert(NodeIDMap::value_type(ID, pNode));
-    }
-}
-
-void Player::initDisplay(const xmlNodePtr xmlNode) {
-    int Height = getDefaultedIntAttr (xmlNode, "height", 0);
-    int Width = getDefaultedIntAttr (xmlNode, "width", 0);
-    SDLDisplayEngine * pSDLDisplayEngine = dynamic_cast<SDLDisplayEngine*>(m_pDisplayEngine);
-    if (pSDLDisplayEngine) {
-        pSDLDisplayEngine->setOGLOptions(m_bUsePOW2Textures, m_YCbCrMode, m_bUseRGBOrder,
-                m_bUsePixelBuffers, m_MultiSampleSamples);
-    }
-    m_pDisplayEngine->init(Width, Height, m_bFullscreen, m_BPP, 
-            m_WindowWidth, m_WindowHeight);
-    m_pDisplayEngine->setGamma(m_Gamma[0], m_Gamma[1], m_Gamma[2]);
-    m_pDisplayEngine->showCursor(m_bShowCursor);
-}
-
 void Player::handleTimers()
 {
     vector<Timeout *>::iterator it;
     m_bInHandleTimers = true;
+    vector<Timeout *> IntervalsFired;
+    
     it = m_PendingTimeouts.begin();
     while (it != m_PendingTimeouts.end() && (*it)->IsReady() && !m_bStopping)
     {
         (*it)->Fire();
-        if (!(*it)->IsInterval()) {
-            delete *it;
-            it = m_PendingTimeouts.erase(it);
-        } else {
-            Timeout* pTempTimeout = *it;
-            it = m_PendingTimeouts.erase(it);
-            addTimeout(pTempTimeout);
+        if (!m_bCurrentTimeoutDeleted) {
+            if ((*it)->IsInterval()) {
+                Timeout* pTempTimeout = *it;
+                it = m_PendingTimeouts.erase(it);
+                m_NewTimeouts.push_back(pTempTimeout);
+            } else {
+                delete *it;
+                it = m_PendingTimeouts.erase(it);
+            }
         }
+        m_bCurrentTimeoutDeleted = false;
     }
     for (it = m_NewTimeouts.begin(); it != m_NewTimeouts.end(); ++it) {
         addTimeout(*it);
     }
     m_NewTimeouts.clear();
-    vector<int>::iterator i;
-    for (i = m_KilledTimeouts.begin(); i != m_KilledTimeouts.end(); ++i) {
-        int id = *i;
-        
-        vector<Timeout *>::iterator it2;
-        for (it2=m_PendingTimeouts.begin(); it2 != m_PendingTimeouts.end(); ++it2) {
-            if (id == (*it2)->GetID()) {
-                delete *it2;
-                m_PendingTimeouts.erase(it2);
-                break;
-            }
-        }
-    }
-    m_KilledTimeouts.clear();
     m_bInHandleTimers = false;
     
 }
@@ -742,28 +791,32 @@ bool Player::handleEvent(Event * pEvent)
                 MouseEvent * pMouseEvent = dynamic_cast<MouseEvent*>(pEvent);
                 DPoint pos(pMouseEvent->getXPosition(), 
                         pMouseEvent->getYPosition());
-                Node * pNode;
-                if (pEvent->getType() != Event::MOUSEOVER &&
-                        pEvent->getType() != Event::MOUSEOUT)
-                {
-                    pNode = m_pRootNode->getElementByPos(pos);
+                NodePtr pNode;
+                if (m_pEventCaptureNode.expired()) {
+                    if (pEvent->getType() != Event::MOUSEOVER &&
+                            pEvent->getType() != Event::MOUSEOUT)
+                    {
+                        pNode = m_pRootNode->getElementByPos(pos);
+                    } else {
+                        pNode = pMouseEvent->getElement();
+                    }
                 } else {
-                    pNode = pMouseEvent->getElement();
-                }
+                    pNode = m_pEventCaptureNode.lock();
+                } 
                 if (pNode != m_pLastMouseNode && 
                         pEvent->getType() != Event::MOUSEOVER &&
                         pEvent->getType() != Event::MOUSEOUT)
                 {
-                    if (pNode) {
-                        createMouseOver(pMouseEvent, Event::MOUSEOVER, pNode);
-                    }
-                    if (m_pLastMouseNode) {
-                        createMouseOver(pMouseEvent, Event::MOUSEOUT, 
+                    if (m_pLastMouseNode && m_pLastMouseNode->getSensitive()) {
+                        sendMouseOver(pMouseEvent, Event::MOUSEOUT, 
                                 m_pLastMouseNode);
                     }
+                    if (pNode && pNode->getSensitive()) {
+                        sendMouseOver(pMouseEvent, Event::MOUSEOVER, pNode);
+                    }
                     m_pLastMouseNode = pNode;
                 }
-                if (pNode) {
+                if (pNode && pNode->getSensitive()) {
                     pNode->handleMouseEvent(pMouseEvent);
                 }
             }
@@ -796,8 +849,8 @@ DisplayEngine * Player::getDisplayEngine() const
     return m_pDisplayEngine;
 }
 
-void Player::createMouseOver(MouseEvent * pOtherEvent, Event::Type Type, 
-                Node * pNode)
+void Player::sendMouseOver(MouseEvent * pOtherEvent, Event::Type Type, 
+                NodePtr pNode)
 {
     MouseEvent * pNewEvent = new MouseEvent(Type,
             pOtherEvent->getLeftButtonState(),
@@ -807,7 +860,7 @@ void Player::createMouseOver(MouseEvent * pOtherEvent, Event::Type Type,
             pOtherEvent->getYPosition(),
             pOtherEvent->getButton());
     pNewEvent->setElement(pNode);
-    m_EventDispatcher.addEvent(pNewEvent);
+    m_EventDispatcher.sendEvent(pNewEvent);
 }
 
 void Player::cleanup() 
@@ -819,11 +872,12 @@ void Player::cleanup()
     }
     m_PendingTimeouts.clear();
     Profiler::get().dumpStatistics();
-    m_pDisplayEngine->deinitRender();
-    m_pDisplayEngine->teardown();
-    delete m_pRootNode;
-    m_pRootNode = 0;
-    m_pLastMouseNode = 0;
+    if (m_pDisplayEngine) {
+        m_pDisplayEngine->deinitRender();
+        m_pDisplayEngine->teardown();
+    }
+    m_pRootNode = AVGNodePtr();
+    m_pLastMouseNode = NodePtr();
     m_IDMap.clear();
     initConfig();
 }
