@@ -45,8 +45,8 @@ RasterNode::RasterNode ()
 {
 }
 
-RasterNode::RasterNode (const xmlNodePtr xmlNode, DivNode * pParent)
-    : Node(xmlNode, pParent),
+RasterNode::RasterNode (const xmlNodePtr xmlNode, Player * pPlayer)
+    : Node(xmlNode, pPlayer),
       m_pSurface(0),
       m_Angle(0),
       m_Pivot(-32767, -32767),
@@ -63,12 +63,14 @@ RasterNode::RasterNode (const xmlNodePtr xmlNode, DivNode * pParent)
 
 RasterNode::~RasterNode()
 {
-    delete m_pSurface;
+    if (m_pSurface) {
+        delete m_pSurface;
+    }
 }
 
-void RasterNode::initVisible()
+void RasterNode::setDisplayEngine(DisplayEngine * pEngine)
 {
-    Node::initVisible();
+    Node::setDisplayEngine(pEngine);
 
     m_bHasCustomPivot = ((m_Pivot.x != -32767) && (m_Pivot.y != -32767));
 
@@ -94,6 +96,15 @@ void RasterNode::initVisible()
     setBlendModeStr(m_sBlendMode);
 }
 
+void RasterNode::disconnect()
+{
+    if (m_pSurface) {
+        delete m_pSurface;
+        m_pSurface = 0;
+    }
+    Node::disconnect();
+}
+
 int RasterNode::getNumVerticesX()
 {
 #ifdef AVG_ENABLE_GL
@@ -198,13 +209,13 @@ void RasterNode::setBlendModeStr(const std::string& sBlendMode)
     }
 }
 
-Node * RasterNode::getElementByPos (const DPoint & pos)
+NodePtr RasterNode::getElementByPos (const DPoint & pos)
 {
     // Node isn't pickable if it's tilted or warped.
     if (fabs(m_Angle)<0.0001 && m_MaxTileSize == IntPoint(-1, -1)) {
         return Node::getElementByPos(pos);
     } else {
-        return 0;
+        return NodePtr();
     }
 }
 
