@@ -36,7 +36,8 @@ class RasterNode: public Node
 {
     public:
         virtual ~RasterNode ();
-        void initVisible();
+        virtual void setDisplayEngine(DisplayEngine * pEngine);
+        virtual void disconnect();
         
         // Warping support.
         int getNumVerticesX();
@@ -69,13 +70,13 @@ class RasterNode: public Node
 #endif
         DisplayEngine::BlendMode getBlendMode() const;
         virtual std::string getTypeStr ();
-        Node * getElementByPos (const DPoint & pos);
+        NodePtr getElementByPos (const DPoint & pos);
 
         Bitmap* getBitmap();
         
     protected:
         RasterNode ();
-        RasterNode (const xmlNodePtr xmlNode, DivNode * pParent);
+        RasterNode (const xmlNodePtr xmlNode, Player * pPlayer);
         DPoint getPivot();
         ISurface * getSurface();
  
