@@ -25,7 +25,6 @@
 #include "SDLMain.h"
 #endif
 
-#include "Region.h"
 #include "Player.h"
 #include "Node.h"
 #include "AVGNode.h"
@@ -47,24 +46,24 @@
 
 #ifdef __APPLE__
 #include <ApplicationServices/ApplicationServices.h>
-#else
+#endif
+#ifdef linux
 #include <X11/Xlib.h>
 #include <X11/extensions/xf86vmode.h>
 #endif
 
-#define XMD_H 1
-#include "GL/gl.h"
-#include "GL/glu.h"
+//#define XMD_H 1
+#include "OGLHelper.h"
+
 
 #ifdef __APPLE__
 #include <OpenGL/OpenGL.h>
-#else
-#define GLX_GLXEXT_PROTOTYPES
-#include "GL/glx.h"
 #endif
 
+#ifdef linux
 #include <sys/ioctl.h>
 #include <sys/fcntl.h>
+#endif
 
 #include <signal.h>
 #include <iostream>
@@ -161,8 +160,6 @@ SDLDisplayEngine::SDLDisplayEngine()
         exit(-1);
     }
     initTranslationTable();
-
-    glproc::init();
 }
 
 SDLDisplayEngine::~SDLDisplayEngine()
@@ -170,22 +167,21 @@ SDLDisplayEngine::~SDLDisplayEngine()
     SDL_QuitSubSystem(SDL_INIT_VIDEO);
 }
 
-void SDLDisplayEngine::init(int width, int height, bool isFullscreen, 
-        int bpp, int WindowWidth, int WindowHeight)
+void SDLDisplayEngine::init(const DisplayParams& DP) 
 {
-    double AspectRatio = double(width)/double(height);
-    if (WindowWidth == 0 && WindowHeight == 0) {
-        m_WindowWidth = width;
-        m_WindowHeight = height;
-    } else if (WindowWidth == 0) {
-        m_WindowWidth = int(WindowHeight*AspectRatio);
-        m_WindowHeight = WindowHeight;
+    double AspectRatio = double(DP.m_Width)/double(DP.m_Height);
+    if (DP.m_WindowWidth == 0 && DP.m_WindowHeight == 0) {
+        m_WindowWidth = DP.m_Width;
+        m_WindowHeight = DP.m_Height;
+    } else if (DP.m_WindowWidth == 0) {
+        m_WindowWidth = int(DP.m_WindowHeight*AspectRatio);
+        m_WindowHeight = DP.m_WindowHeight;
     } else {
-        m_WindowWidth = WindowWidth;
-        m_WindowHeight = int(WindowWidth/AspectRatio);
+        m_WindowWidth = DP.m_WindowWidth;
+        m_WindowHeight = int(DP.m_WindowWidth/AspectRatio);
     }
     
-    switch (bpp) {
+    switch (DP.m_BPP) {
         case 32:
             safeSetAttribute( SDL_GL_RED_SIZE, 8 );
             safeSetAttribute( SDL_GL_GREEN_SIZE, 8 );
@@ -211,7 +207,7 @@ void SDLDisplayEngine::init(int width, int height, bool isFullscreen,
             safeSetAttribute( SDL_GL_BUFFER_SIZE, 15 );
             break;
         default:
-            AVG_TRACE(Logger::ERROR, "Unsupported bpp " << bpp <<
+            AVG_TRACE(Logger::ERROR, "Unsupported bpp " << DP.m_BPP <<
                     "in SDLDisplayEngine::init()");
             exit(-1);
     }
@@ -225,16 +221,19 @@ void SDLDisplayEngine::init(int width, int height, bool isFullscreen,
     }
 
     unsigned int Flags = SDL_OPENGL;
-    if (isFullscreen) {
+    if (DP.m_bFullscreen) {
         Flags |= SDL_FULLSCREEN;
     }
-    m_pScreen = SDL_SetVideoMode(m_WindowWidth, m_WindowHeight, bpp, Flags);
+    m_pScreen = SDL_SetVideoMode(m_WindowWidth, m_WindowHeight, DP.m_BPP, Flags);
     if (!m_pScreen) {
         AVG_TRACE(Logger::ERROR, "Setting SDL video mode failed: " 
                 << SDL_GetError() <<". (width=" << m_WindowWidth << ", height=" 
-                << m_WindowHeight << ", bpp=" << bpp << ").");
+                << m_WindowHeight << ", bpp=" << DP.m_BPP << ", multisamplesamples="
+                << m_MultiSampleSamples << ").");
         exit(-1);
     }   
+    glproc::init();
+    
     SDL_WM_SetCaption("AVG Renderer", 0);
     calcRefreshRate();
 
@@ -254,11 +253,18 @@ void SDLDisplayEngine::init(int width, int height, bool isFullscreen,
         glDisable(GL_MULTISAMPLE);
         OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, "init: glDisable(GL_MULTISAMPLE);");
     }
+    setGamma(DP.m_Gamma[0], DP.m_Gamma[1], DP.m_Gamma[2]);
+    showCursor(DP.m_bShowCursor);
+    if (DP.m_Framerate == 0) {
+        setVBlankRate(DP.m_VBRate);
+    } else {
+        setFramerate(DP.m_Framerate);
+    }
 
     checkYCbCrSupport();
 
-    m_Width = width;
-    m_Height = height;
+    m_Width = DP.m_Width;
+    m_Height = DP.m_Height;
     initInput();
     // SDL sets up a signal handler we really don't want.
     signal(SIGSEGV, SIG_DFL);
@@ -327,7 +333,7 @@ void SDLDisplayEngine::logConfig()
 static ProfilingZone PrepareRenderProfilingZone("  Root node: prepareRender");
 static ProfilingZone RootRenderProfilingZone("  Root node: render");
 
-void SDLDisplayEngine::render(AVGNode * pRootNode, bool bRenderEverything)
+void SDLDisplayEngine::render(AVGNodePtr pRootNode, bool bRenderEverything)
 {
     if (!m_bEnableCrop && pRootNode->getCropSetting()) {
         m_bEnableCrop = true;
@@ -338,7 +344,6 @@ void SDLDisplayEngine::render(AVGNode * pRootNode, bool bRenderEverything)
         OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
                 "setClipPlane: glEnable()");
     }
-
     {
         ScopeTimer Timer(PrepareRenderProfilingZone);
         pRootNode->prepareRender(0, pRootNode->getAbsViewport());
@@ -448,8 +453,8 @@ void SDLDisplayEngine::setClipPlane(double Eqn[4], int WhichPlane)
 void SDLDisplayEngine::clip()
 {
     
-    DRect rc = m_ClipRects.back();
-    if (m_bEnableCrop) {       
+    if (m_bEnableCrop && !m_ClipRects.empty()) {       
+        DRect rc = m_ClipRects.back();
         double yEqn[4] = {0.0, 1.0, 0.0, -rc.tl.y};
         //    yEqn[3] = -rc.tl.y;
         setClipPlane(yEqn, GL_CLIP_PLANE0);
@@ -524,6 +529,11 @@ OGLShaderPtr SDLDisplayEngine::getYCbCr420pShader()
     return m_pYCbCrShader;
 }
 
+OGLShaderPtr SDLDisplayEngine::getYCbCrJ420pShader()
+{
+    return m_pYCbCrJShader;
+}
+
 void SDLDisplayEngine::showCursor (bool bShow)
 {
     if (bShow) {
@@ -572,7 +582,7 @@ void SDLDisplayEngine::checkYCbCrSupport()
         )
     {
         m_YCbCrMode = OGL_SHADER;
-        string sProgram =
+        string sProgramInit = 
 #ifdef __APPLE__
             "#define textureRect texture2DRect\n"
             "#define samplerRect sampler2DRect\n"
@@ -580,7 +590,9 @@ void SDLDisplayEngine::checkYCbCrSupport()
             "uniform samplerRect YTexture;\n"
             "uniform samplerRect CbTexture;\n"
             "uniform samplerRect CrTexture;\n"
-            "\n"
+            "\n";
+        string sProgram =
+            sProgramInit + 
             "void main(void)\n"
             "{\n"
             "  vec3 YCbCr;\n"
@@ -588,15 +600,32 @@ void SDLDisplayEngine::checkYCbCrSupport()
             "  YCbCr.g = textureRect(CbTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
             "  YCbCr.b = textureRect(CrTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
             "  vec3 RGB;"
-            "  RGB = YCbCr*mat3(1.16, 0.0 , 1.60,\n"
+            "  RGB = YCbCr*mat3(1.16,  0.0,   1.60,\n"
             "                   1.16, -0.39, -0.81,\n"
-            "                   1.16, 2.01, 0.0 );\n"
+            "                   1.16,  2.01,  0.0 );\n"
             "  gl_FragColor = vec4(RGB,gl_Color.a);\n"
             "}\n"
             ;
         m_pYCbCrShader = OGLShaderPtr(new OGLShader(sProgram));
-    } else 
-    if (queryOGLExtension("GL_MESA_ycbcr_texture") &&
+
+        sProgram =
+            sProgramInit + 
+            "void main(void)\n"
+            "{\n"
+            "  vec3 YCbCr;\n"
+            "  YCbCr.r = textureRect(YTexture, gl_TexCoord[0].st).a;\n"
+            "  YCbCr.g = textureRect(CbTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
+            "  YCbCr.b = textureRect(CrTexture, (gl_TexCoord[0].st)/2.0).a-0.5;\n"
+            "  vec3 RGB;"
+            "  RGB = YCbCr*mat3(1,  0.0  , 1.40,\n"
+            "                   1, -0.34, -0.71,\n"
+            "                   1,  1.77,  0.0 );\n"
+            "  gl_FragColor = vec4(RGB,gl_Color.a);\n"
+            "}\n"
+            ;
+        m_pYCbCrJShader = OGLShaderPtr(new OGLShader(sProgram));
+
+    } else if (queryOGLExtension("GL_MESA_ycbcr_texture") &&
             (m_DesiredYCbCrMode == OGL_SHADER || m_DesiredYCbCrMode == OGL_MESA))
     {
         m_YCbCrMode = OGL_MESA;
@@ -678,7 +707,7 @@ bool SDLDisplayEngine::vbWait(int rate) {
 #ifdef linux
         case VB_SGI: {
                 unsigned int count;
-                int err = glXWaitVideoSyncSGI(rate, m_VBMod, &count);
+                int err = glproc::WaitVideoSyncSGI(rate, m_VBMod, &count);
                 OGLErrorCheck(AVG_ERR_VIDEO_GENERAL, 
                         "VBlank::glXWaitVideoSyncSGI");
                 if (err) {
@@ -745,7 +774,8 @@ void SDLDisplayEngine::calcRefreshRate() {
         if (value) {
             CFNumberGetValue(value, kCFNumberIntType, &s_RefreshRate);
             if (s_RefreshRate < 1.0) {
-                AVG_TRACE(Logger::CONFIG, "This seems to be a TFT screen.");
+                AVG_TRACE(Logger::CONFIG, 
+                        "This seems to be a TFT screen, assuming 60 Hz refresh rate.");
                 s_RefreshRate = 60;
             }
         } else {
@@ -756,7 +786,7 @@ void SDLDisplayEngine::calcRefreshRate() {
         AVG_TRACE(Logger::WARNING, 
                 "Apple refresh rate calculation (CGDisplayCurrentMode) failed");
     }
-#else 
+#elif defined linux 
     Display * display = XOpenDisplay(0);
     int PixelClock;
     XF86VidModeModeLine mode_line;
@@ -1236,6 +1266,7 @@ int SDLDisplayEngine::getMaxTexSize()
     return m_MaxTexSize;
 }
 
+// This is what OpenGL calls InternalFormat
 int SDLDisplayEngine::getOGLDestMode(PixelFormat pf)
 {
     switch (pf) {
@@ -1302,14 +1333,22 @@ int SDLDisplayEngine::getOGLSrcMode(PixelFormat pf)
 
 int SDLDisplayEngine::getOGLPixelType(PixelFormat pf)
 {
-    if (pf == YCbCr422) {
-        if (getYCbCrMode() == DisplayEngine::OGL_MESA) {
-            return GL_UNSIGNED_SHORT_8_8_REV_MESA;
-        } else {
-            return GL_UNSIGNED_SHORT_8_8_APPLE;
-        }
-    } else {
-        return GL_UNSIGNED_BYTE;
+    switch (pf) {
+        case YCbCr422:
+            if (getYCbCrMode() == DisplayEngine::OGL_MESA) {
+                return GL_UNSIGNED_SHORT_8_8_REV_MESA;
+            } else {
+#ifdef __i386__
+                return GL_UNSIGNED_SHORT_8_8_REV_APPLE;
+#else
+                return GL_UNSIGNED_SHORT_8_8_APPLE;
+#endif 
+            }
+        case B8G8R8X8:
+        case B8G8R8A8:
+            return GL_UNSIGNED_INT_8_8_8_8_REV;
+        default:
+            return GL_UNSIGNED_BYTE;
     }
 }
 
@@ -1322,18 +1361,6 @@ OGLMemoryMode SDLDisplayEngine::getMemoryModeSupported()
         {
             m_MemoryMode = PBO;
             AVG_TRACE(Logger::CONFIG, "Using pixel buffer objects.");
-/*
-#ifndef __APPLE__
-        } else if (queryGLXExtension("GLX_MESA_allocate_memory")) {
-            // Disabled because it's buggy.
-            s_MemoryMode = MESA;
-            s_AllocMemMESAProc = (PFNGLXALLOCATEMEMORYMESAPROC)
-                    glXGetProcAddressARB((const GLubyte*)"glXAllocateMemoryMESA");
-            s_FreeMemMESAProc = (PFNGLXFREEMEMORYMESAPROC)
-                    glXGetProcAddressARB((const GLubyte*)"glXFreeMemoryMESA");
-            AVG_TRACE(Logger::CONFIG, "Using MESA extension to allocate AGP memory.");
-#endif
-*/
         } else {
             m_MemoryMode = OGL;
             AVG_TRACE(Logger::CONFIG, "Not using GL memory extensions.");
@@ -1361,3 +1388,4 @@ void SDLDisplayEngine::setOGLOptions(bool bUsePOW2Textures, YCbCrMode DesiredYCb
 }
 
 }
+
