@@ -19,7 +19,7 @@ class LinearAnim(SimpleAnim):
     def __init__(self, node, attrName, duration, startValue, endValue, useInt, onStop):
         SimpleAnim.__init__(self, node, attrName, duration, useInt, onStop)
         g_Player.setTimeout(duration, self.__stop)
-        self.interval = g_Player.setInterval(10, self.__step)
+        self.interval = g_Player.setInterval(1, self.__step)
         self.__startValue = startValue
         self.__endValue = endValue
         self.__done = 0
@@ -27,6 +27,8 @@ class LinearAnim(SimpleAnim):
     def __step(self):
         if not(self.__done):
             part = ((time.time()-self.startTime)/self.duration)*1000
+            if part > 1.0:
+                part = 1.0
             curValue = self.__startValue+(self.__endValue-self.__startValue)*part
             if self.useInt:
                 curValue = int(curValue+0.5)
@@ -43,7 +45,7 @@ class SplineAnim(SimpleAnim):
             startValue, startSpeed, endValue, endSpeed, useInt, onStop):
         SimpleAnim.__init__(self, node, attrName, duration, useInt, onStop)
         g_Player.setTimeout(duration, self.__stop)
-        self.interval = g_Player.setInterval(10, self.__step)
+        self.interval = g_Player.setInterval(1, self.__step)
         self.__startValue = startValue+0.0
         self.__startSpeed = startSpeed
         self.__endValue = endValue
