@@ -22,7 +22,9 @@
 void export_bitmap();
 void export_raster();
 void export_event();
+#ifndef WIN32
 void export_devices();
+#endif
 
 #include "../base/Logger.h"
 #include "../base/Exception.h"
@@ -33,6 +35,7 @@ void export_devices();
 
 #include <boost/python.hpp>
 #include <boost/version.hpp>
+#include <boost/shared_ptr.hpp>
 
 using namespace boost::python;
 using namespace avg;
@@ -47,6 +50,8 @@ BOOST_PYTHON_MODULE(avg)
 #if (BOOST_VERSION / 100000) > 1 || ((BOOST_VERSION / 100) % 1000) >= 33
     register_exception_translator<Exception>(exception_translator);
 #endif
+    register_ptr_to_python< DivNodePtr >();
+    register_ptr_to_python< AVGNodePtr >();
 
     class_<Logger>("Logger", 
             "Interface to the logger used by the avg player. Enables the setting\n"
@@ -110,10 +115,12 @@ BOOST_PYTHON_MODULE(avg)
         .def_readonly("APP", &Logger::APP)
     ;
 
-    export_devices();
+#ifndef WIN32
+     export_devices();
+#endif
     export_event();
-    
-    class_<Node, boost::noncopyable>("Node",
+
+    class_<Node, boost::shared_ptr<Node>, boost::noncopyable>("Node",
             "Base class for all elements in the avg tree.\n"
             "Properties:\n"
             "    id: A unique identifier that can be used to reference the node (ro).\n"
@@ -129,12 +136,22 @@ BOOST_PYTHON_MODULE(avg)
             "    sensitive: A node only reacts to events if sensitive is true.",
             no_init)
         .def("getParent", &Node::getParent,
-                return_internal_reference<>(),
                 "getParent() -> Node\n\n"
                 "Returns the container (AVGNode or DivNode) the node is in. For\n"
                 "the root node, returns None.\n")
-        .add_property("id", make_function(&Node::getID, 
-                return_value_policy<copy_const_reference>()))
+        .def("setEventCapture", &Node::setEventCapture,
+                "setEventCapture() -> None\n\n"
+                "Sets up event capturing so that all mouse events are sent to this node\n"
+                "regardless of the mouse cursor position. If the node doesn't handle the\n"
+                "event, it propagates to its parent normally. Useful for the\n"
+                "implementation of user interface elements such as scroll bars. Only one\n"
+                "node can capture the mouse at any one time. Normal mouse operation can\n"
+                "be restored by calling releaseEventCapture().\n")
+        .def("releaseEventCapture", &Node::releaseEventCapture,
+                "releaseEventCapture() -> None\n\n"
+                "Restores normal nouse operation after a call to setEventCapture()\n")
+        .add_property("id", make_function(&Node::getID,
+                return_value_policy<copy_const_reference>()), &Node::setID)
         .add_property("x", &Node::getX, &Node::setX)
         .add_property("y", &Node::getY, &Node::setY)
         .add_property("width", &Node::getWidth, &Node::setWidth)
@@ -156,7 +173,6 @@ BOOST_PYTHON_MODULE(avg)
         .def("getNumChildren", &DivNode::getNumChildren,
                 "getNumChildren() -> numChildren\n\n")
         .def("getChild", &DivNode::getChild, 
-                return_value_policy<reference_existing_object>(),
                 "getChild(i) -> Node\n\n"
                 "Returns the ith child in z-order.")
         .def("addChild", &DivNode::addChild,
@@ -203,11 +219,22 @@ BOOST_PYTHON_MODULE(avg)
             "    saturation: The saturation the image should have. (ro, deprecated)\n"
             "    rotation: The current angle the viewer is looking at in radians.\n"
             "    maxrotation: The maximum angle the viewer can look at.\n")
-        .add_property("href", make_function(&PanoImage::getFilename,
-                return_value_policy<copy_const_reference>()))
-        .add_property("sensorwidth", &PanoImage::getSensorWidth)
-        .add_property("sensorheight", &PanoImage::getSensorHeight)
-        .add_property("focallength", &PanoImage::getFocalLength)
+        .def("getScreenPosFromPanoPos", &PanoImage::getScreenPosFromPanoPos,
+                "getScreenPosFromPanoPos(panoPos) -> pos\n\n"
+                "Converts a position in panorama image pixels to pixels in coordinates\n"
+                "relative to the node, taking into account the current rotation angle.\n")
+        .def("getScreenPosFromAngle", &PanoImage::getScreenPosFromAngle,
+                "getScreenPosFromAngle(angle) -> pos\n\n"
+                "Converts panorama angle to pixels in coordinates\n"
+                "relative to the node, taking into account the current rotation angle.\n")
+        .add_property("href", make_function(&PanoImage::getHRef, 
+                return_value_policy<copy_const_reference>()), &PanoImage::setHRef)
+        .add_property("sensorwidth", &PanoImage::getSensorWidth, 
+                &PanoImage::setSensorWidth)
+        .add_property("sensorheight", &PanoImage::getSensorHeight, 
+                &PanoImage::setSensorHeight)
+        .add_property("focallength", &PanoImage::getFocalLength, 
+                &PanoImage::setFocalLength)
         .add_property("hue", &PanoImage::getHue)
         .add_property("saturation", &PanoImage::getSaturation)
         .add_property("rotation", &PanoImage::getRotation, &PanoImage::setRotation)
@@ -230,6 +257,7 @@ BOOST_PYTHON_MODULE(avg)
 
     class_<TestHelper>("TestHelper", "", no_init)
         .def("getNumDifferentPixels", &TestHelper::getNumDifferentPixels, "")
+        .def("fakeMouseEvent", &TestHelper::fakeMouseEvent, "")
     ;
 
     class_<Player>("Player", 
@@ -287,7 +315,6 @@ BOOST_PYTHON_MODULE(avg)
                 return_value_policy<reference_existing_object>(),
                 "")
         .def("createNode", &Player::createNodeFromXmlString,
-                return_value_policy<manage_new_object>(),
                 "createNode(xml) -> Node\n\n"
                 "Creates a new Node from an xml string. This node can be used as\n"
                 "parameter to DivNode::addChild(). BROKEN!")
@@ -324,12 +351,10 @@ BOOST_PYTHON_MODULE(avg)
                 "OpenGL. Showing the DirectFB mouse cursor seems to expose some\n"
                 "issue with DirectFB.)")
         .def("getElementByID", &Player::getElementByID,
-                return_value_policy<reference_existing_object>(),
                 "getElementByID(id) -> Node\n\n"
                 "Returns an element in the avg tree. The id corresponds to the id\n"
                 "attribute of the node.")
         .def("getRootNode", &Player::getRootNode,
-                return_value_policy<reference_existing_object>(),
                 "getRootNode() -> AVGNode\n\n"
                 "Returns the outermost element in the avg tree.")
         .def("getFramerate", &Player::getFramerate,
