@@ -3,7 +3,8 @@
 import unittest
 
 import sys, syslog, time, os
-
+import tempfile
+sys.path += ['../python/.libs', '../python']
 import avg
 import anim
 
@@ -17,22 +18,29 @@ class LoggerTestCase(unittest.TestCase):
     def test(self):
         self.Log = avg.Logger.get()
         self.Log.setCategories(self.Log.APP |
-                  self.Log.WARNING | 
+                  self.Log.WARNING
 #                  self.Log.PROFILE |
 #                  self.Log.PROFILE_LATEFRAMES |
-                  self.Log.CONFIG |
-                  self.Log.MEMORY | 
+#                  self.Log.CONFIG |
+#                  self.Log.MEMORY | 
 #                  self.Log.BLTS    |
-                  self.Log.EVENTS
+#                  self.Log.EVENTS
+#                  self.Log.EVENTS2
                   )
+        myTempFile = os.path.join(tempfile.gettempdir(), "testavg.log")
         try:
-            os.remove("/tmp/testavg.log")
+            os.remove(myTempFile)
         except OSError:
             pass
-        self.Log.setFileDest("/tmp/testavg.log")
+        self.Log.setFileDest(myTempFile)
         self.Log.trace(self.Log.APP, "Test file log entry.")
-        stats = os.stat("/tmp/testavg.log")
-        self.assert_(stats.st_size == 50)
+        readLog = file(myTempFile, "r").readlines()
+        self.assert_(len(readLog) == 1)
+        myBaseLine = "APP: Test file log entry."
+        self.assert_(readLog[0].find(myBaseLine) >= 0)
+        stats = os.stat(myTempFile)
+        # Windows text files have two chars for linefeed
+        self.assert_(stats.st_size in [50, 51])
         
         self.Log.setSyslogDest(syslog.LOG_USER, syslog.LOG_CONS)
         self.Log.trace(self.Log.APP, "Test syslog entry.")
@@ -53,22 +61,25 @@ class AVGTestCase(unittest.TestCase):
         self.__testFuncName = testFuncName
         self.Log = avg.Logger.get()
         unittest.TestCase.__init__(self, testFuncName)
-    def setUp(self):
+    def setUpVideo(self):
         Player.setDisplayEngine(self.__engine)
         Player.setResolution(0, 0, 0, self.__bpp)
         Player.setOGLOptions(UsePOW2Textures, YCbCrMode, UseRGBOrder, UsePixelBuffers, 1)
+    def setUp(self):
+        self.setUpVideo()
         print "-------- ", self.__testFuncName, " --------"
     def start(self, filename, actions):
         self.assert_(Player.isPlaying() == 0)
         Player.loadFile(filename)
         self.actions = actions
         self.curFrame = 0
-        Player.setInterval(1, self.nextAction)
-        Player.setFramerate(1000)
+        Player.setInterval(0, self.nextAction)
+        Player.setFramerate(100)
         Player.play()
         self.assert_(Player.isPlaying() == 0)
     def nextAction(self):
         self.actions[self.curFrame]()
+#        print (self.curFrame)
         self.curFrame += 1
     def compareImage(self, fileName, warn):
         global CREATE_BASELINE_IMAGES
@@ -76,21 +87,26 @@ class AVGTestCase(unittest.TestCase):
         if CREATE_BASELINE_IMAGES:
             Bmp.save(BASELINE_DIR+"/"+fileName+".png")
         else:
-            BaselineBmp = avg.Bitmap(BASELINE_DIR+"/"+fileName+".png")
-            NumPixels = Player.getTestHelper().getNumDifferentPixels(Bmp, BaselineBmp)
-            if (NumPixels > 20):
-                if ourSaveDifferences:
-                    Bmp.save(RESULT_DIR+"/"+fileName+".png")
-                    BaselineBmp.save(RESULT_DIR+"/"+fileName+"_baseline.png")
-                    Bmp.subtract(BaselineBmp)
-                    Bmp.save(RESULT_DIR+"/"+fileName+"_diff.png")
-                self.Log.trace(self.Log.WARNING, "Image compare: "+str(NumPixels)+
-                        " bright pixels.")
-                if warn:
-                    self.Log.trace(self.Log.WARNING, "Image "+fileName
-                            +" differs from original.")
-                else:
-                    self.assert_(False)
+            try:
+                BaselineBmp = avg.Bitmap(BASELINE_DIR+"/"+fileName+".png")
+                NumPixels = Player.getTestHelper().getNumDifferentPixels(Bmp, BaselineBmp)
+                if (NumPixels > 20):
+                    if ourSaveDifferences:
+                        Bmp.save(RESULT_DIR+"/"+fileName+".png")
+                        BaselineBmp.save(RESULT_DIR+"/"+fileName+"_baseline.png")
+                        Bmp.subtract(BaselineBmp)
+                        Bmp.save(RESULT_DIR+"/"+fileName+"_diff.png")
+                    self.Log.trace(self.Log.WARNING, "Image compare: "+str(NumPixels)+
+                            " bright pixels.")
+                    if warn:
+                        self.Log.trace(self.Log.WARNING, "Image "+fileName
+                                +" differs from original.")
+                    else:
+                        self.assert_(False)
+            except RuntimeError:
+                Bmp.save(RESULT_DIR+"/"+fileName+".png")
+                self.Log.trace(self.Log.WARNING, "Could not load image "+fileName+".png")
+                self.assert_(False)
 
 def keyUp():
     print "keyUp"
@@ -115,40 +131,89 @@ def dumpMouseEvent():
     print "  position: "+str(Event.x)+","+str(Event.y)
     print "  node: "+Event.node.id
 
+mainMouseDownCalled = False
+mouseMove1Called = False
+mouseUp1Called = False
+mouseDown1Called = False
+mouseOver1Called = False
+mouseOut1Called = False
+divMouseDownCalled = False
+mouseDown2Called = False
+obscuredMouseDownCalled = False
+deactMouseDownCalled = False
+deactMouseOverLate = False
+deactMouseOverCalled = False
 
 def mainMouseUp():
-    print "mainMouseUp"
-#    dumpMouseEvent()
+    global mainMouseUpCalled
+    mainMouseUpCalled = True
 
 def mainMouseDown():
-    print "mainMouseDown"
-#    dumpMouseEvent()
+    global mainMouseDownCalled
+    mainMouseDownCalled = True
 
-def onMouseMove():
-    print "onMouseMove"
-#    dumpMouseEvent()
+def onMouseMove1():
+    print "onMouseMove1"
 
-def onMouseUp():
-    print "onMouseUp"
-#    dumpMouseEvent()
+def onMouseUp1():
+    global mouseUp1Called
+    mouseUp1Called = True
 
-def onMouseOver():
-    print "onMouseOver"
-    dumpMouseEvent()
+def onMouseDown1():
+    global mouseDown1Called
+    mouseDown1Called = True
 
-def onMouseOut():
-    print "onMouseOut"
-    dumpMouseEvent()
+def onMouseOver1():
+    global mouseOver1Called
+    mouseOver1Called = True
 
-def onMouseDown():
-    print "onMouseDown"
-    Player.getElementByID("mouseover1").active=0
-    Player.getElementByID("rightdiv").active=0
-    dumpMouseEvent()
+def onMouseOut1():
+    global mouseOut1Called
+    mouseOut1Called = True
+
+def onDivMouseDown():
+    global divMouseDownCalled
+    divMouseDownCalled = True
+
+def onMouseDown2():
+    global mouseDown2Called
+    mouseDown2Called = True
+
+def onObscuredMouseDown():
+    global obscuredMouseDownCalled
+    obscuredMouseDownCalled = True
 
 def onErrMouseOver():
     undefinedFunction()
 
+def onDeactMouseDown():
+    global deactMouseDownCalled
+    deactMouseDownCalled = True
+
+def onDeactMouseOver():
+    global deactMouseDownCalled
+    global deactMouseOverLate
+    global deactMouseOverCalled
+    deactMouseOverLate = deactMouseDownCalled
+    deactMouseOverCalled = True
+
+def onDeactMouseMove():
+    print("move")
+
+def onDeactMouseOut():
+    print("out")
+
+mainCaptureMouseDownCalled = False
+captureMouseDownCalled = False
+
+def onMainCaptureMouseDown():
+    global mainCaptureMouseDownCalled
+    mainCaptureMouseDownCalled = True
+
+def onCaptureMouseDown():
+    global captureMouseDownCalled
+    captureMouseDownCalled = True
+
 class PlayerTestCase(AVGTestCase):
     def __init__(self, testFuncName, engine, bpp):
         AVGTestCase.__init__(self, testFuncName, engine, bpp)
@@ -161,6 +226,8 @@ class PlayerTestCase(AVGTestCase):
             Bmp = node.getBitmap()
             self.assert_(Bmp.getSize() == (65,65))
             self.assert_(Bmp.getFormat() == avg.R8G8B8 or Bmp.getFormat() == avg.B8G8R8)
+        Player.showCursor(0)
+        Player.showCursor(1)
         self.start("image.avg",
                 (lambda: self.compareImage("testimg", False), 
                  getBitmap,
@@ -168,6 +235,8 @@ class PlayerTestCase(AVGTestCase):
                  lambda: self.compareImage("testimgload", False),
                  lambda: Player.setGamma(0.7, 0.7, 0.7),
                  lambda: Player.setGamma(1.0, 1.0, 1.0),
+                 lambda: Player.showCursor(0),
+                 lambda: Player.showCursor(1),
                  Player.stop))
     def testError(self):
         Player.loadFile("image.avg")
@@ -199,10 +268,97 @@ class PlayerTestCase(AVGTestCase):
     def testEvents(self):
         def getMouseState():
             Event = Player.getMouseState()
-            Event.x
+        def testInactiveDiv():
+            Player.getElementByID("div1").active = False
+            Helper.fakeMouseEvent(avg.MOUSEBUTTONDOWN, True, False, False,
+                70, 70, 1)
+        Helper = Player.getTestHelper()
+        global mainMouseDownCalled
+        global mouseMove1Called
+        global mouseUp1Called
+        global mouseDown1Called
+        global mouseOver1Called
+        global mouseOut1Called
+        global divMouseDownCalled
+        global mouseDown2Called
+        global obscuredMouseDownCalled
+        global deactMouseOverCalled
+        global deactMouseOverLate
+        global deactMouseDownCalled
+
         self.start("events.avg", 
-                (getMouseState, 
-                 lambda: self.compareImage("testEvents", False), 
+                (lambda: self.compareImage("testEvents", False),
+                 lambda: Helper.fakeMouseEvent(avg.MOUSEBUTTONDOWN, True, False, False,
+                        10, 10, 1),
+                 lambda: self.assert_(mouseDown1Called and mouseOver1Called 
+                        and mainMouseDownCalled),
+                 getMouseState,
+                 lambda: Helper.fakeMouseEvent(avg.MOUSEBUTTONUP, True, False, False,
+                        12, 12, 1),
+                 lambda: self.assert_(mouseUp1Called and mainMouseUpCalled),
+                 lambda: Helper.fakeMouseEvent(avg.MOUSEBUTTONDOWN, True, False, False,
+                        70, 70, 1),
+                 lambda: self.assert_(mouseDown2Called and divMouseDownCalled and 
+                        mouseOut1Called and not(obscuredMouseDownCalled)),
+                 testInactiveDiv,
+                 lambda: self.assert_(obscuredMouseDownCalled),
+                 # Test if deactivation between mouse click and mouse out works.
+                 lambda: Helper.fakeMouseEvent(avg.MOUSEBUTTONDOWN, True, False, False,
+                        70, 10, 1),
+                 lambda: self.assert_(deactMouseOverCalled and not(deactMouseOverLate)),
+                 # XXX
+                 # - errMouseOver
+                 Player.stop))
+    def testEventCapture(self):
+        def captureEvent():
+            global captureMouseDownCalled
+            Helper = Player.getTestHelper()
+            captureMouseDownCalled = False
+            mainCaptureMouseDownCalled = False
+            Player.getElementByID("img1").setEventCapture()
+            Helper.fakeMouseEvent(avg.MOUSEBUTTONDOWN, True, False, False,
+                    100, 10, 1)
+        def noCaptureEvent():
+            global captureMouseDownCalled
+            Helper = Player.getTestHelper()
+            captureMouseDownCalled = False
+            mainCaptureMouseDownCalled = False
+            Player.getElementByID("img1").releaseEventCapture()
+            Helper.fakeMouseEvent(avg.MOUSEBUTTONDOWN, True, False, False,
+                    100, 10, 1)
+        global captureMouseDownCalled
+        global mainCaptureMouseDownCalled
+        Helper = Player.getTestHelper()
+        self.start("eventcapture.avg",
+                (lambda: Helper.fakeMouseEvent(avg.MOUSEBUTTONDOWN, True, False, False,
+                        10, 10, 1),
+                 lambda: self.assert_(captureMouseDownCalled),
+                 captureEvent,
+                 lambda: self.assert_(captureMouseDownCalled and 
+                        mainCaptureMouseDownCalled),
+                 noCaptureEvent,
+                 lambda: self.assert_(not(captureMouseDownCalled) and 
+                        mainCaptureMouseDownCalled),
+                 Player.stop))
+    def testTimeouts(self):
+        self.timeout1called = False
+        self.timeout2called = False
+        def timeout1():
+            Player.clearInterval(self.timeout1ID)
+            Player.clearInterval(self.timeout2ID)
+            self.timeout1called = True
+        def timeout2():
+            self.timeout2called = True
+        def wait():
+            pass
+        def setupTimeouts():
+            self.timeout1ID = Player.setTimeout(0, timeout1)
+            self.timeout2ID = Player.setTimeout(1, timeout2)
+        self.start("image.avg",
+                (setupTimeouts,
+                 wait,
+                 lambda: self.assert_(self.timeout1called),
+                 lambda: self.assert_(not(self.timeout2called)),
                  Player.stop))
     def testEventErr(self):
         Player.loadFile("errevent.avg")
@@ -221,13 +377,31 @@ class PlayerTestCase(AVGTestCase):
                  lambda: self.compareImage("testHugeImage1", False),
                  Player.stop))
     def testPanoImage(self):
+        def changeProperties():
+            node = Player.getElementByID("pano")
+            node.sensorheight=10
+            node.sensorwidth=15
+            node.focallength=25
+        def loadImage():
+            node = Player.getElementByID("pano")
+            node.href = "rgb24-65x65.png"
         self.start("panoimage.avg",
                 (lambda: self.compareImage("testPanoImage", False),
+                 lambda: time.sleep,
+                 changeProperties,
+                 loadImage,
                  Player.stop))
     def testBroken(self):
-        Player.loadFile("noxml.avg")
-        Player.loadFile("noavg.avg")
-        Player.loadFile("noavg2.avg")
+        def testBrokenFile(filename):
+            exceptionRaised = False
+            try:
+                Player.loadFile(filename)
+            except RuntimeError:
+                exceptionRaised = True
+            self.assert_(exceptionRaised)
+        testBrokenFile("noxml.avg")
+        testBrokenFile("noavg.avg")
+        testBrokenFile("noavg2.avg")
     def testMove(self):
         def moveit():
             node = Player.getElementByID("nestedimg1")
@@ -348,6 +522,7 @@ class PlayerTestCase(AVGTestCase):
         def changeFont():
             node = Player.getElementByID("cbasetext")
             node.font = "Times New Roman"
+            node.height = 0
             node.size = 30
         def changeFont2():
             node = Player.getElementByID("cbasetext")
@@ -369,12 +544,12 @@ class PlayerTestCase(AVGTestCase):
                  Player.stop))
     def testVideo(self):
         def seek():
-            Player.getElementByID("clogo2").seekToFrame(180)
+            Player.getElementByID("clogo2").seekToFrame(100)
         def foo():
             pass
         def newHRef():
             node = Player.getElementByID("clogo2")
-            node.href = "test.m2v"
+            node.href = "h264-48x48.h264"
             node.play()
         def move():
             node = Player.getElementByID("clogo2")
@@ -387,7 +562,6 @@ class PlayerTestCase(AVGTestCase):
                 (lambda: self.compareImage("testVideo1", False),
                  lambda: Player.getElementByID("clogo2").play(),
                  seek,
-#                 foo,
                  lambda: self.compareImage("testVideo2", False),
                  lambda: Player.getElementByID("clogo2").pause(),
                  lambda: self.compareImage("testVideo3", False),
@@ -407,9 +581,14 @@ class PlayerTestCase(AVGTestCase):
                  lambda: self.compareImage("testVideo9", False),
                  Player.stop))
     def testAnim(self):
+        def onStart():
+            Player.setTimeout(10, startAnim)
+            Player.setTimeout(380, startSplineAnim)
+            Player.setTimeout(800, lambda: self.compareImage("testAnim3", False))
+            Player.setTimeout(850, Player.stop)
         def startAnim():
             def onStop():
-                self.__animStopped = 1
+                self.__animStopped = True
             self.compareImage("testAnim1", False)
             anim.fadeOut(Player.getElementByID("nestedimg2"), 200)
             Player.getElementByID("nestedimg1").opacity = 0
@@ -417,58 +596,182 @@ class PlayerTestCase(AVGTestCase):
             anim.LinearAnim(Player.getElementByID("nestedimg1"), "x", 
                     200, 0, 100, 0, onStop)
         def startSplineAnim():
-            self.assert_(self.__animStopped == 1)
+            self.assert_(self.__animStopped)
             self.compareImage("testAnim2", False)
             anim.SplineAnim(Player.getElementByID("mainimg"), "x", 
                     200, 100, -400, 10, 0, 0, None)
             anim.SplineAnim(Player.getElementByID("mainimg"), "y", 
                     200, 100, 0, 10, -400, 1, None)
+        self.__animStopped = False
         anim.init(Player)
         Player.loadFile("avg.avg")
-        Player.setTimeout(10, startAnim)
-        Player.setTimeout(390, startSplineAnim)
-        Player.setTimeout(800, lambda: self.compareImage("testAnim3", False))
-        Player.setTimeout(850, Player.stop)
+        Player.setTimeout(1, onStart)
         Player.setVBlankFramerate(1)
         Player.play()
         
-#    def createNodes(self):
-#        node=Player.createNode("<image href='rgb24.png'/>")
-#        node.x = 10
-#        node.y = 20
-#        node.z = 2
-#        node.opacity = 0.333
-#        node.angle = 0.1
-#        node.blendmode = "add"
-##        print node.toXML()
-#        self.rootNode.addChild(node)
-##        nodeCopy = node
-##        self.rootNode.addChild(nodeCopy)
-#        node = Player.createNode("<video href='test.m1v'/>")
-#        self.rootNode.addChild(node)
-#        node = Player.createNode("<words text='Lorem ipsum dolor'/>")
-#        self.rootNode.addChild(node)
-#        node.size = 18
-#        node.font = "times new roman"
-#        node.parawidth = 200
-#        node = Player.createNode("<div><image href='rgb24.png'/></div>")
-#        node.getChild(0).x=10
-#        node.x=10
-#        self.rootNode.addChild(node)
-#    def deleteNodes(self):
-#        for i in range(self.rootNode.getNumChildren()-1,0):
-##            print ("Deleting node #"+i);
-#            self.rootNode.removeChild(i)
-#    def testDynamics(self):
-#        Player.loadFile("image.avg")
-#        self.rootNode = Player.getRootNode()
-#        print self.rootNode.indexOf(Player.getElementByID("mainimg"));
-#        print self.rootNode.indexOf(Player.getElementByID("testtiles"));
-#        self.createNodes()
-#        Player.setTimeout(250, self.deleteNodes)
-#        Player.setTimeout(500, self.createNodes)
-#        Player.play()
-   
+    def testImgDynamics(self):
+        def createImg():
+            node = Player.createNode("<image href='rgb24-64x64.png'/>")
+            node.id = "newImage"
+            node.x = 10
+            node.y = 20
+            node.opacity = 0.666
+            node.angle = 0.1
+            node.blendmode = "add"
+#            print node.toXML()
+            self.rootNode = Player.getRootNode()
+            self.rootNode.addChild(node)
+            exceptionRaised=False
+            try:
+                node.id = "bork"
+            except RuntimeError:
+                exceptionRaised=True
+            self.assert_(exceptionRaised)
+            self.assert_(self.rootNode.indexOf(Player.getElementByID("newImage")) == 0)
+        def removeImg():
+            self.imgNode = Player.getElementByID("newImage")
+            self.rootNode.removeChild(
+                    self.rootNode.indexOf(self.imgNode))
+            self.assert_(Player.getElementByID("newImage") == None)
+        def reAddImg():
+            self.rootNode.addChild(self.imgNode)
+        Player.loadFile("empty.avg")
+        createImg()
+        Player.stop()
+        self.setUpVideo()
+        self.start("empty.avg",
+                (createImg,
+                 lambda: self.compareImage("testImgDynamics1", False),
+                 removeImg,
+                 lambda: self.compareImage("testImgDynamics2", False),
+                 reAddImg,
+                 lambda: self.compareImage("testImgDynamics3", False),
+                 Player.stop))
+
+    def testVideoDynamics(self):
+        def createVideo():
+            node = Player.createNode("<video id='newVideo' href='mpeg1-48x48.mpg'/>")
+            self.rootNode = Player.getRootNode()
+            self.rootNode.addChild(node)
+            node.play()
+        def removeVideo():
+            self.videoNode = Player.getElementByID("newVideo")
+            self.rootNode.removeChild(self.rootNode.indexOf(self.videoNode))
+        def reAddVideo():
+            self.rootNode.addChild(self.videoNode)
+            exceptionRaised = False
+            try:
+                self.rootNode.addChild(self.videoNode)
+            except RuntimeError:
+                exceptionRaised = True
+            self.assert_(exceptionRaised)
+            self.videoNode.play()
+        def foo():
+            pass
+        Player.loadFile("empty.avg")
+        createVideo()
+        Player.stop()
+        self.setUpVideo()
+        self.start("empty.avg",
+                (createVideo,
+                 lambda: self.compareImage("testVideoDynamics1", False),
+                 removeVideo,
+                 lambda: self.compareImage("testVideoDynamics2", False),
+                 reAddVideo,
+                 lambda: self.compareImage("testVideoDynamics3", False),
+                 Player.stop))
+    def testWordsDynamics(self):
+        def createWords():
+            node = Player.createNode("<words id='newWords' text='test'/>")
+            node.font="times new roman"
+            node.size=12
+            node.parawidth=200
+            node.x=10
+            self.rootNode = Player.getRootNode()
+            self.rootNode.addChild(node)
+        def removeWords():
+            self.wordsNode = Player.getElementByID("newWords")
+            self.rootNode.removeChild(self.rootNode.indexOf(self.wordsNode))
+        def reAddWords():
+            self.rootNode.addChild(self.wordsNode)
+            self.wordsNode.text='test2'
+        Player.loadFile("empty.avg")
+        createWords()
+        Player.stop()
+        self.setUpVideo()
+        self.start("empty.avg",
+                (createWords,
+                 lambda: self.compareImage("testWordsDynamics1", True),
+                 removeWords,
+                 lambda: self.compareImage("testWordsDynamics2", True),
+                 reAddWords,
+                 lambda: self.compareImage("testWordsDynamics3", True),
+                 Player.stop))
+    def testCameraDynamics(self):
+        def createCamera():
+            node = Player.createNode("<camera id='newCamera' device='/dev/video1394/0' mode='640x480_RGB' framerate='15'/>")
+            self.rootNode = Player.getRootNode()
+            self.rootNode.addChild(node)
+        def removeCamera():
+            self.cameraNode = Player.getElementByID("newCamera")
+            self.rootNode.removeChild(self.rootNode.indexOf(self.cameraNode))
+        def reAddCamera():
+            self.rootNode.addChild(self.cameraNode)
+        Player.loadFile("empty.avg")
+        createCamera()
+        Player.stop()
+        self.setUpVideo()
+        self.start("empty.avg",
+                (createCamera,
+                 removeCamera,
+                 reAddCamera,
+                 Player.stop))
+    def testPanoDynamics(self):
+        def createPano():
+            node = Player.createNode("<panoimage id='newPano' href='panoimage.png' sensorwidth='4.60' sensorheight='3.97' focallength='12' width='160' height='120'/>")
+            self.rootNode = Player.getRootNode()
+            self.rootNode.addChild(node)
+        def removePano():
+            self.panoNode = Player.getElementByID("newPano")
+            self.rootNode.removeChild(self.rootNode.indexOf(self.panoNode))
+        def reAddPano():
+            self.rootNode.addChild(self.panoNode)
+        Player.loadFile("empty.avg")
+        createPano()
+        Player.stop()
+        self.setUpVideo()
+        self.start("empty.avg",
+                (createPano,
+                 lambda: self.compareImage("testPanoDynamics1", False),
+                 removePano,
+                 lambda: self.compareImage("testPanoDynamics2", False),
+                 reAddPano,
+                 lambda: self.compareImage("testPanoDynamics3", False),
+                 Player.stop))
+    def testDivDynamics(self):
+        def createDiv():
+            node = Player.createNode("<div id='newDiv'><image id='nestedImg' href='rgb24-64x64.png'/></div>")
+            self.rootNode = Player.getRootNode()
+            self.rootNode.addChild(node)
+        def removeDiv():
+            self.divNode = Player.getElementByID("newDiv")
+            self.rootNode.removeChild(self.rootNode.indexOf(self.divNode))
+        def reAddDiv():
+            node = Player.createNode("<image id='img2' href='rgb24-64x64.png' x='64'/>")
+            self.divNode.addChild(node)
+            self.rootNode.addChild(self.divNode)
+        Player.loadFile("empty.avg")
+        createDiv()
+        Player.stop()
+        self.setUpVideo()
+        self.start("empty.avg",
+                (createDiv,
+                 lambda: self.compareImage("testDivDynamics1", False),
+                 removeDiv,
+                 lambda: self.compareImage("testDivDynamics2", False),
+                 reAddDiv,
+                 lambda: self.compareImage("testDivDynamics3", False),
+                 Player.stop))
             
 def playerTestSuite(engine, bpp):
     def rmBrokenDir():
@@ -480,7 +783,7 @@ def playerTestSuite(engine, bpp):
             try:
                 os.mkdir(RESULT_DIR)
             except OSError:
-                # This can happen on make distcheck - permission denied...
+                # This can happen on make distcheck (permission denied...)
                 global ourSaveDifferences
                 ourSaveDifferences = False
     rmBrokenDir()
@@ -491,6 +794,8 @@ def playerTestSuite(engine, bpp):
     suite.addTest(PlayerTestCase("testExceptionInTimeout", engine, bpp))
     suite.addTest(PlayerTestCase("testInvalidImageFilename", engine, bpp))
     suite.addTest(PlayerTestCase("testEvents", engine, bpp))
+    suite.addTest(PlayerTestCase("testEventCapture", engine, bpp))
+    suite.addTest(PlayerTestCase("testTimeouts", engine, bpp))
     suite.addTest(PlayerTestCase("testEventErr", engine, bpp))
     suite.addTest(PlayerTestCase("testHugeImage", engine, bpp))
     suite.addTest(PlayerTestCase("testPanoImage", engine, bpp))
@@ -503,6 +808,12 @@ def playerTestSuite(engine, bpp):
     suite.addTest(PlayerTestCase("testWords", engine, bpp))
     suite.addTest(PlayerTestCase("testVideo", engine, bpp))
     suite.addTest(PlayerTestCase("testAnim", engine, bpp))
+    suite.addTest(PlayerTestCase("testImgDynamics", engine, bpp))
+    suite.addTest(PlayerTestCase("testVideoDynamics", engine, bpp))
+    suite.addTest(PlayerTestCase("testWordsDynamics", engine, bpp))
+    suite.addTest(PlayerTestCase("testPanoDynamics", engine, bpp))
+    suite.addTest(PlayerTestCase("testCameraDynamics", engine, bpp))
+    suite.addTest(PlayerTestCase("testDivDynamics", engine, bpp))
     return suite
 
 def completeTestSuite(engine, bpp):
@@ -563,9 +874,11 @@ if not(customOGLOptions):
 
 SrcDir = os.getenv("srcdir",".")
 os.chdir(SrcDir)
-print(os.getcwd())
-os.system("whoami")
 Player = avg.Player()
 runner = unittest.TextTestRunner()
-runner.run(completeTestSuite(engine, bpp))
+rc = runner.run(completeTestSuite(engine, bpp))
+if rc.wasSuccessful():
+    sys.exit(0)
+else:
+    sys.exit(1)
 
